@@ -32,7 +32,7 @@ protected:
   QString netlist();
   QString vhdlCode(int);
   QString verilogCode(int);
-  QString spice_netlist(bool isXyce = false);
+  QString spice_netlist(spicecompat::SpiceDialect dialect = spicecompat::SPICEDefault);
 };
 
 #endif
