@@ -18,16 +18,16 @@
 #ifndef TR_SIM_H
 #define TR_SIM_H
 
-#include "component.h"
+#include "simulation.h"
 
 
-class TR_Sim : public Component  {
+class TR_Sim : public qucs::component::SimulationComponent {
 public:
   TR_Sim();
   ~TR_Sim();
   Component* newOne();
   static Element* info(QString&, char* &, bool getNewOne=false);
-  QString spice_netlist(bool isXyce = false);
+  QString spice_netlist(spicecompat::SpiceDialect dialect = spicecompat::SPICEDefault);
   void recreate(Schematic*);
 };
 
