@@ -54,6 +54,7 @@
 #include <QDateTime>
 #include <QPainter>
 #include <QDebug>
+#include <QtAlgorithms>
 
 Diagram::Diagram(int _cx, int _cy) {
     cx = _cx;
@@ -93,77 +94,72 @@ Diagram::Diagram(int _cx, int _cy) {
 }
 
 Diagram::~Diagram() {
+    qDeleteAll(Graphs);
+    qDeleteAll(Arcs);
+    qDeleteAll(Lines);
+    qDeleteAll(Texts);
 }
 
 /*!
    Paint function for most diagrams (cartesian, smith, polar, ...)
 */
-void Diagram::paint(ViewPainter *p) {
+void Diagram::paint(QPainter *p) {
     paintDiagram(p);
     paintMarkers(p);
 }
 
-void Diagram::paintDiagram(ViewPainter *p) {
-    // paint all lines
-    for (qucs::Line *pl: Lines) {
-        p->Painter->setPen(pl->style);
-        p->drawLine(cx + pl->x1, cy - pl->y1, cx + pl->x2, cy - pl->y2);
+void Diagram::paintDiagram(QPainter *painter) {
+    painter->save();
+
+    painter->translate(cx, cy);
+    painter->save();
+
+    for (qucs::Line* line : Lines) {
+        painter->setPen(line->penHint());
+        painter->drawLine(QLineF{line->x1, - line->y1, line->x2, - line->y2});
     }
 
-    // paint all arcs (1 pixel larger to compensate for strange circle method)
-    for (qucs::Arc *pa: Arcs) {
-        p->Painter->setPen(pa->style);
-        p->drawArc(cx + pa->x, cy - pa->y, pa->w, pa->h, pa->angle, pa->arclen);
+    for (qucs::Arc* arc : Arcs) {
+        painter->setPen(arc->penHint());
+        painter->drawArc(QRectF{arc->x, - arc->y, arc->w, arc->h}, arc->angle, arc->arclen);
     }
 
-    // draw all graphs
-    for (Graph *pg: Graphs)
-        pg->paint(p, cx, cy);
+    painter->scale(1.0, -1.0); // make Y-axis grow upwards
+    for (Graph *pg: Graphs) {
+        pg->paint(painter);
+    }
+    painter->restore();  // to translated(cx, cy) with no negative y-scale
 
-    // keep track of painter state
-    p->Painter->save();
-
-    // write whole text (axis label inclusively)
-    QTransform wm = p->Painter->worldTransform();
     for (Text *pt: Texts) {
-        p->Painter->setWorldTransform(
-                QTransform(pt->mCos, -pt->mSin, pt->mSin, pt->mCos,
-                           p->DX + float(cx + pt->x) * p->Scale,
-                           p->DY + float(cy - pt->y) * p->Scale));
+        painter->save();
 
-        p->Painter->setPen(pt->Color);
-        p->Painter->drawText(0, 0, pt->s);
+        painter->setPen(pt->Color);
+        painter->translate(pt->x, -pt->y);
+        painter->rotate(pt->angle());
+        painter->drawText(0, 0, pt->s);
+
+        painter->restore();
     }
-    p->Painter->setWorldTransform(wm);
-    p->Painter->setWorldMatrixEnabled(false);
-
-    // restore painter state
-    p->Painter->restore();
-
 
     if (isSelected) {
-        int x_, y_;
-        float fx_, fy_;
-        p->map(cx, cy - y2, x_, y_);
-        fx_ = float(x2) * p->Scale + 10;
-        fy_ = float(y2) * p->Scale + 10;
+        QRectF bounds(0, -y2, x2, y2);
+        painter->setPen(QPen(Qt::darkGray, 3));
+        painter->drawRect(bounds.marginsAdded(QMargins{5, 5, 5, 5}));
 
-        p->Painter->setPen(QPen(Qt::darkGray, 3));
-        p->Painter->drawRect(x_ - 5, y_ - 5, lround(fx_), lround(fy_));
-        p->Painter->setPen(QPen(Qt::darkRed, 2));
-        p->drawResizeRect(cx, cy - y2);  // markers for changing the size
-        p->drawResizeRect(cx, cy);
-        p->drawResizeRect(cx + x2, cy - y2);
-        p->drawResizeRect(cx + x2, cy);
+        misc::draw_resize_handle(painter, bounds.topLeft());
+        misc::draw_resize_handle(painter, bounds.bottomLeft());
+        misc::draw_resize_handle(painter, bounds.bottomRight());
+        misc::draw_resize_handle(painter, bounds.topRight());
     }
+    painter->restore();
 }
 
-void Diagram::paintMarkers(ViewPainter *p, bool paintAll) {
+void Diagram::paintMarkers(QPainter *p, bool paintAll) {
     // draw markers last, so they are at the top of painting layers
     for (Graph *pg: Graphs)
         for (Marker *pm: pg->Markers)
             if (paintAll || (pm->Type & 1)) {
-                pm->paint(p, cx, cy);
+                pm->paint(p);
             }
 }
 
@@ -222,6 +218,7 @@ void Diagram::createAxisLabels() {
 
     QStringList used_kernels, used_simulations;
     for (const auto pg: Graphs) {
+      if (!pg->Var.contains("/")) continue; // Qucsator data
         QString kernel_name = pg->Var.section('/', 0, 0);
         QString var_name = pg->Var;
         auto p = var_name.indexOf('/');
@@ -854,15 +851,17 @@ int Graph::loadDatFile(const QString &fileName) {
 
     Info.setFile(file);
     if (g->lastLoaded.isValid())
-        if (g->lastLoaded.currentMSecsSinceEpoch() >
-            Info.lastModified().currentMSecsSinceEpoch()) //Millisecond resulution is needed for tuning
+        if (g->lastLoaded.toMSecsSinceEpoch() >=
+            Info.lastModified().toMSecsSinceEpoch()) //Millisecond resulution is needed for tuning
             return 1;    // dataset unchanged -> no update necessary
+    qDebug() << "Loading data from " << Info.canonicalFilePath();
 
+    qDeleteAll(g->mutable_axes());
+    g->mutable_axes().clear();
     g->countY = 0;
-    g->mutable_axes().clear(); // HACK
-    if (g->cPointsY) {
+    if (g->cPointsY != nullptr) {
         delete[] g->cPointsY;
-        g->cPointsY = 0;
+        g->cPointsY = nullptr;
     }
     if (Variable.isEmpty()) return 0;
 
@@ -1850,97 +1849,51 @@ bool Diagram::calcAxisScale(Axis *Axis, double &GridNum, double &zD,
  \param[out]  z      - screen coordinate where the first grid is placed
  \param[out]  zD     - number where the first grid is placed
  \param[out]  zDstep - number increment from one grid to the next
- \param[out]  coor   - scale factor for calculate screen coordinate
+ \param[out]  corr   - scale factor for calculate screen coordinate
 
  \todo use this as example to document other methods
 */
 bool Diagram::calcAxisLogScale(Axis *Axis, int &z, double &zD,
                                double &zDstep, double &corr, int len) {
-    if (fabs(Axis->max - Axis->min) < 1e-200) { // if max = min, double difference
-        Axis->max *= 10.0;
-        Axis->min /= 10.0;
-    }
-    Axis->low = Axis->min;
-    Axis->up = Axis->max;
+    bool mirror = false;
 
-    if (!Axis->autoScale) {
-        Axis->low = Axis->limit_min;
-        Axis->up = Axis->limit_max;
-    }
-
-
-    bool mirror = false, mirror2 = false;
-    double tmp;
-    if (Axis->up < 0.0) {   // for negative values
-        tmp = Axis->low;
-        Axis->low = -Axis->up;
-        Axis->up = -tmp;
-        mirror = true;
-    }
-
-    double Base, Expo;
     if (Axis->autoScale) {
-        if (mirror) {   // set back values ?
-            tmp = Axis->min;
-            Axis->min = -Axis->max;
-            Axis->max = -tmp;
-        }
-
-        Expo = floor(log10(Axis->max));
-        Base = Axis->max / pow(10.0, Expo);
-        if (Base > 3.0001) Axis->up = pow(10.0, Expo + 1.0);
-        else if (Base < 1.0001) Axis->up = pow(10.0, Expo);
-        else Axis->up = 3.0 * pow(10.0, Expo);
-
-        Expo = floor(log10(Axis->min));
-        Base = Axis->min / pow(10.0, Expo);
-        if (Base < 2.999) Axis->low = pow(10.0, Expo);
-        else if (Base > 9.999) Axis->low = pow(10.0, Expo + 1.0);
-        else Axis->low = 3.0 * pow(10.0, Expo);
-
-        corr = double(len) / log10(Axis->up / Axis->low);
-
-        z = 0;
-        zD = Axis->low;
-        zDstep = pow(10.0, Expo);
-
-        if (mirror) {   // set back values ?
-            tmp = Axis->min;
-            Axis->min = -Axis->max;
-            Axis->max = -tmp;
-        }
-    } else {   // user defined limits
-        if (Axis->up < Axis->low) {
-            tmp = Axis->low;
-            Axis->low = Axis->up;
-            Axis->up = tmp;
-            mirror2 = true;
-        }
-
-        Expo = floor(log10(Axis->low));
-        Base = ceil(Axis->low / pow(10.0, Expo));
-        zD = Base * pow(10.0, Expo);
-        zDstep = pow(10.0, Expo);
-        if (zD > 9.5 * zDstep) zDstep *= 10.0;
-
-        corr = double(len) / log10(Axis->up / Axis->low);
-        z = lround(corr * log10(zD / Axis->low)); // int(..) implies floor(..)
-
-        if (mirror2) {   // set back values ?
-            tmp = Axis->low;
-            Axis->low = Axis->up;
-            Axis->up = tmp;
-        }
+	double minExp = floor(log10(Axis->min));
+	double maxExp = ceil(log10(Axis->max));
+	if ( minExp == maxExp ) {
+	    minExp -= 1;
+	    maxExp += 1;
+	}
+	Axis->low = pow(10.0, minExp);
+	Axis->up = pow(10.0, maxExp);
+    } else {
+	Axis->low = Axis->limit_min;
+	Axis->up = Axis->limit_max;
     }
 
+    if (Axis->up < Axis->low) {
+	double tmp = Axis->low;
+	Axis->low = Axis->up;
+	Axis->up = tmp;
+	mirror = true;
+    }
+
+    double Expo = floor(log10(Axis->low));
+    double Base = ceil(Axis->low / pow(10.0, Expo));
+    zD = Base * pow(10.0, Expo);
+    zDstep = pow(10.0, Expo);
+    if (zD > 9.5 * zDstep) zDstep *= 10.0;
+
+    corr = double(len) / log10(Axis->up / Axis->low);
+    z = lround(corr * log10(zD / Axis->low)); // int(..) implies floor(..)
+
     if (mirror) {   // set back values ?
-        tmp = Axis->low;
-        Axis->low = -Axis->up;
-        Axis->up = -tmp;
+	double tmp = Axis->low;
+	Axis->low = Axis->up;
+	Axis->up = tmp;
     }
 
-    if (mirror == mirror2) return false;
-    else return true;
+    return mirror;
 }
 
 // --------------------------------------------------------------
@@ -1956,8 +1909,8 @@ bool Diagram::calcYAxis(Axis *Axis, int x0) {
     bool back = false;
     if (Axis->log) {
         if (Axis->autoScale) {
-            if (Axis->max * Axis->min <= 0.0) return false;  // invalid
-        } else if (Axis->limit_min * Axis->limit_max <= 0.0) return false;  // invalid
+            if (Axis->max <= 0.0 or Axis->min <= 0.0) return false;  // invalid
+        } else if (Axis->limit_min <= 0.0 or Axis->limit_max <= 0.0) return false;  // invalid
 
         back = calcAxisLogScale(Axis, z, zD, zDstep, corr, y2);
 
