@@ -25,6 +25,7 @@
 #include "rect3ddiagram.h"
 #include "main.h"
 #include "misc.h"
+#include "settings.h"
 
 #include <cmath>
 #include <assert.h>
@@ -47,6 +48,7 @@
 #include <QHeaderView>
 #include <QDir>
 #include <QDebug>
+#include <QtAlgorithms>
 
 #define CROSS3D_SIZE   30
 #define WIDGET3D_SIZE  2*CROSS3D_SIZE
@@ -82,16 +84,16 @@ private:
 
     Painter.setPen(QPen(Qt::red,2));
     Painter.drawLine(CROSS3D_SIZE, CROSS3D_SIZE,
-		     int(CROSS3D_SIZE * (1.0+cxx)),
-		     int(CROSS3D_SIZE * (1.0-cyx)));
+         int(CROSS3D_SIZE * (1.0+cxx)),
+         int(CROSS3D_SIZE * (1.0-cyx)));
     Painter.setPen(QPen(Qt::green,2));
     Painter.drawLine(CROSS3D_SIZE, CROSS3D_SIZE,
-		     int(CROSS3D_SIZE * (1.0-cxy)),
-		     int(CROSS3D_SIZE * (1.0-cyy)));
+         int(CROSS3D_SIZE * (1.0-cxy)),
+         int(CROSS3D_SIZE * (1.0-cyy)));
     Painter.setPen(QPen(Qt::blue,2));
     Painter.drawLine(CROSS3D_SIZE, CROSS3D_SIZE,
-		     int(CROSS3D_SIZE * (1.0+cxz)),
-		     int(CROSS3D_SIZE * (1.0+cyz)));
+         int(CROSS3D_SIZE * (1.0+cxz)),
+         int(CROSS3D_SIZE * (1.0+cyz)));
   };
 };
 
@@ -110,15 +112,14 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
 {
     setAttribute(Qt::WA_DeleteOnClose);
   Diag = d;
-  Graphs.setAutoDelete(true);
   copyDiagramGraphs();   // make a copy of all graphs
   if(parent){
-	  const Schematic* s = dynamic_cast<const Schematic*>(parent);
-	  assert(s);
-	  QFileInfo Info(s->DocName);
-      defaultDataSet = Info.absolutePath() + QDir::separator() + s->DataSet;
+    const Schematic* s = dynamic_cast<const Schematic*>(parent);
+    assert(s);
+    QFileInfo Info(s->getDocName());
+      defaultDataSet = Info.absolutePath() + QDir::separator() + s->getDataSet();
   }else{
-	  defaultDataSet = "unknown";
+    defaultDataSet = "unknown";
   }
   setWindowTitle(tr("Edit Diagram Properties"));
   changed = false;
@@ -155,7 +156,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
     NameZ = tr("z-Axis");
   }
 
-  
+
   all = new QVBoxLayout(this); // to provide necessary size
   QTabWidget *t = new QTabWidget();
   all->addWidget(t);
@@ -241,7 +242,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
       PropertyBox->addItem(tr("arrows"));
     }
     connect(PropertyBox, SIGNAL(activated(int)),
-			 SLOT(slotSetGraphStyle(int)));
+      SLOT(slotSetGraphStyle(int)));
     Box2Layout->setStretchFactor(new QWidget(Box2), 5); // stretchable placeholder
 
     Label2 = new QLabel(tr("Thickness:"));
@@ -251,7 +252,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
     Property2->setValidator(ValInteger);
     Property2->setMaximumWidth(25);
     Property2->setMaxLength(2);
-    Property2->setText("0");
+    Property2->setText(_settings::Get().item<QString>("DefaultGraphLineWidth"));
 
     if((Diag->Name=="Rect") || (Diag->Name=="PS") || (Diag->Name=="SP") || (Diag->Name=="Curve")) {
       Label4 = new QLabel(tr("y-Axis:"));
@@ -267,7 +268,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
   }
   if(Property2) {
     connect(Property2, SIGNAL(textChanged(const QString&)),
-			SLOT(slotSetProp2(const QString&)));
+      SLOT(slotSetProp2(const QString&)));
 
     Label1->setEnabled(false);
     PropertyBox->setEnabled(false);
@@ -389,7 +390,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
       GridColorButt = new QPushButton("        ",Tab2);
       connect(GridColorButt, SIGNAL(clicked()), SLOT(slotSetGridColor()));
       gp->addWidget(GridColorButt, Row,1);
-      Row++;    
+      Row++;
       misc::setPickerColor(GridColorButt, Diag->GridPen.color());
 
       GridLabel2 = new QLabel(tr("Grid Style: "), Tab2);
@@ -489,7 +490,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
         rotationX->setMaximumWidth(40);
         gp->addWidget(rotationX, Row,2);
         connect(rotationX, SIGNAL(textChanged(const QString&)),
-			   SLOT(slotEditRotX(const QString&)));
+          SLOT(slotEditRotX(const QString&)));
         Row++;
 
         QLabel *LabelRotY = new QLabel(tr("Rotation around y-Axis:"), Tab2);
@@ -511,7 +512,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
         rotationY->setMaximumWidth(40);
         gp->addWidget(rotationY, Row,2);
         connect(rotationY, SIGNAL(textChanged(const QString&)),
-			   SLOT(slotEditRotY(const QString&)));
+          SLOT(slotEditRotY(const QString&)));
         Row++;
 
         QLabel *LabelRotZ = new QLabel(tr("Rotation around z-Axis:"), Tab2);
@@ -533,13 +534,13 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
         rotationZ->setMaximumWidth(40);
         gp->addWidget(rotationZ, Row,2);
         connect(rotationZ, SIGNAL(textChanged(const QString&)),
-			   SLOT(slotEditRotZ(const QString&)));
+          SLOT(slotEditRotZ(const QString&)));
         Row++;
 
         gp->addWidget(new QLabel(tr("2D-projection:"), Tab2), Row,0);
         DiagCross = new Cross3D(((Rect3DDiagram*)Diag)->rotX,
-				((Rect3DDiagram*)Diag)->rotY,
-				((Rect3DDiagram*)Diag)->rotZ, Tab2);
+          ((Rect3DDiagram*)Diag)->rotY,
+          ((Rect3DDiagram*)Diag)->rotZ, Tab2);
         gp->addWidget(DiagCross, Row,1);
 
         // transfer the diagram properties to the dialog
@@ -805,7 +806,7 @@ DiagramDialog::DiagramDialog(Diagram *d, QWidget *parent, Graph *currentGraph)
   if(ColorButt) {
     if(!currentGraph) {
       QColor selectedColor(DefaultColors[GraphList->count()%NumDefaultColors]);
-      QString stylesheet = QString("QPushButton {background-color: %1};").arg(selectedColor.name());
+      QString stylesheet = QStringLiteral("QPushButton {background-color: %1};").arg(selectedColor.name());
       ColorButt->setStyleSheet(stylesheet);
       misc::setPickerColor(ColorButt, selectedColor);
     }
@@ -988,7 +989,7 @@ void DiagramDialog::slotTakeVar(QTableWidgetItem* Item)
         g->Color = misc::getWidgetBackgroundColor(ColorButt);
         g->Thick = Property2->text().toInt();
         QColor selectedColor(DefaultColors[GraphList->count()%NumDefaultColors]);
-        QString stylesheet = QString("QPushButton {background-color: %1};").arg(selectedColor.name());
+        QString stylesheet = QStringLiteral("QPushButton {background-color: %1};").arg(selectedColor.name());
         ColorButt->setStyleSheet(stylesheet);
         misc::setPickerColor(ColorButt, selectedColor);
         if(g->Var.right(3) == ".Vb")   // harmonic balance output ?
@@ -1012,7 +1013,7 @@ void DiagramDialog::slotTakeVar(QTableWidgetItem* Item)
       g->numMode   = PropertyBox->currentIndex();
     }
 
-    Graphs.append(g);
+    Graphs.emplace_back(g);
     changed = true;
     toTake  = true;
   //}
@@ -1037,7 +1038,7 @@ void DiagramDialog::slotSelectGraph(QListWidgetItem *item)
     return;
   }
 
-  SelectGraph (Graphs.at (GraphList->currentRow()));
+  SelectGraph (Graphs.at(GraphList->currentRow()).get());
 }
 
 /*!
@@ -1053,7 +1054,7 @@ void DiagramDialog::SelectGraph(Graph *g)
   if(Diag->Name != "Tab") {
     if(Diag->Name != "Truth") {
       Property2->setText(QString::number(g->Thick));
-      QString stylesheet = QString("QPushButton {background-color: %1};").arg(g->Color.name());
+      QString stylesheet = QStringLiteral("QPushButton {background-color: %1};").arg(g->Color.name());
       ColorButt->setStyleSheet(stylesheet);
       misc::setPickerColor(ColorButt,g->Color);
       PropertyBox->setCurrentIndex(g->Style);
@@ -1090,7 +1091,7 @@ void DiagramDialog::slotDeleteGraph()
   if(i < 0) return;   // return, if no item selected
 
   GraphList->takeItem(i);
-  Graphs.remove(i);
+  Graphs.erase(std::next(Graphs.begin(), i));
 
   int k=0;
   if (GraphList->count()!=0) {
@@ -1107,7 +1108,7 @@ void DiagramDialog::slotDeleteGraph()
   if(Diag->Name != "Tab") {
     if(Diag->Name != "Truth") {
       QColor selectedColor(DefaultColors[GraphList->count()%NumDefaultColors]);
-      QString stylesheet = QString("QPushButton {background-color: %1};").arg(selectedColor.name());
+      QString stylesheet = QStringLiteral("QPushButton {background-color: %1};").arg(selectedColor.name());
       ColorButt->setStyleSheet(stylesheet);
       misc::setPickerColor(ColorButt,selectedColor);
       Property2->setText("0");
@@ -1159,7 +1160,7 @@ void DiagramDialog::slotNewGraph()
     g->Precision = Property2->text().toInt();
     g->numMode   = PropertyBox->currentIndex();
   }
-  Graphs.append(g);
+  Graphs.emplace_back(g);
   changed = true;
   toTake  = false;
 }
@@ -1220,7 +1221,7 @@ void DiagramDialog::slotApply()
             changed = true;
         }
     }
-    
+
     if(GridOn) if(Diag->xAxis.GridOn != GridOn->isChecked()) {
       Diag->xAxis.GridOn = GridOn->isChecked();
       Diag->yAxis.GridOn = GridOn->isChecked();
@@ -1262,7 +1263,7 @@ void DiagramDialog::slotApply()
     }
 
     if((Diag->Name == "Smith") || (Diag->Name == "ySmith") ||
-				  (Diag->Name == "PS"))
+          (Diag->Name == "PS"))
       if(stopY->text().toDouble() < 1.0)
         stopY->setText("1");
 
@@ -1349,12 +1350,13 @@ void DiagramDialog::slotApply()
 
   }   // of "if(Diag->Name != "Tab")"
 
+  qDeleteAll(Diag->Graphs);
   Diag->Graphs.clear();   // delete the graphs
-  Graphs.setAutoDelete(false);
-  for(Graph *pg = Graphs.first(); pg != 0; pg = Graphs.next())
-    Diag->Graphs.append(pg);  // transfer the new graphs to diagram
+
+  for (std::unique_ptr<Graph>& graph : Graphs) {
+    Diag->Graphs.append(graph.release());  // transfer the new graphs to diagram
+  }
   Graphs.clear();
-  Graphs.setAutoDelete(true);
 
   Diag->loadGraphData(defaultDataSet);
   ((Schematic*)parent())->viewport()->repaint();
@@ -1385,15 +1387,14 @@ void DiagramDialog::slotSetColor()
 {
   QColor c = QColorDialog::getColor(misc::getWidgetBackgroundColor(ColorButt),this);
   if(!c.isValid()) return;
-  QString stylesheet = QString("QPushButton {background-color: %1};").arg(c.name());
+  QString stylesheet = QStringLiteral("QPushButton {background-color: %1};").arg(c.name());
   ColorButt->setStyleSheet(stylesheet);
   misc::setPickerColor(ColorButt,c);
 
   int i = GraphList->currentRow();
   if(i < 0) return;   // return, if no item selected
 
-  Graph *g = Graphs.at(i);
-  g->Color = c;
+  Graphs.at(i)->Color = c;
   changed = true;
   toTake  = false;
 }
@@ -1416,8 +1417,7 @@ void DiagramDialog::slotResetToTake(const QString& s)
   int i = GraphList->currentRow();
   if(i < 0) return;   // return, if no item selected
 
-  Graph *g = Graphs.at(i);
-  g->Var = s;
+  Graphs.at(i)->Var = s;
   // \todo GraphList->changeItem(s, i);   // must done after the graph settings !!!
   changed = true;
   toTake  = false;
@@ -1432,7 +1432,7 @@ void DiagramDialog::slotSetProp2(const QString& s)
   int i = GraphList->currentRow();
   if(i < 0) return;   // return, if no item selected
 
-  Graph *g = Graphs.at(i);
+  Graph *g = Graphs.at(i).get();
   if(Diag->Name == "Tab") g->Precision = s.toInt();
   else  g->Thick = s.toInt();
   changed = true;
@@ -1447,8 +1447,7 @@ void DiagramDialog::slotSetNumMode(int Mode)
   int i = GraphList->currentRow();
   if(i < 0) return;   // return, if no item selected
 
-  Graph *g = Graphs.at(i);
-  g->numMode = Mode;
+  Graphs.at(i)->numMode = Mode;
   changed = true;
   toTake  = false;
 }
@@ -1480,7 +1479,7 @@ void DiagramDialog::slotSetGraphStyle(int style)
   int i = GraphList->currentRow();
   if(i < 0) return;   // return, if no item selected
 
-  Graph *g = Graphs.at(i);
+  Graph *g = Graphs.at(i).get();
   g->Style = toGraphStyle(style);
   assert(g->Style!=GRAPHSTYLE_INVALID);
   changed = true;
@@ -1493,7 +1492,7 @@ void DiagramDialog::slotSetGraphStyle(int style)
 void DiagramDialog::copyDiagramGraphs()
 {
   for (Graph *pg : Diag->Graphs)
-    Graphs.append(pg->sameNewOne());
+    Graphs.emplace_back(pg->sameNewOne());
 }
 
 /*!
@@ -1504,8 +1503,7 @@ void DiagramDialog::slotSetYAxis(int axis)
   int i = GraphList->currentRow();
   if(i < 0) return;   // return, if no item selected
 
-  Graph *g = Graphs.at(i);
-  g->yAxisNo = axis;
+  Graphs.at(i)->yAxisNo = axis;
   changed = true;
   toTake  = false;
 }
