@@ -60,27 +60,29 @@ void Module::registerModule (QString category, pInfoFunc info) {
 // Component registration using a category name and the appropriate
 // function returning a components instance object.
 void Module::registerComponent(QString category, pInfoFunc info) {
-    Module *m = new Module();
-    m->info = info;
-    m->category = category;
 
-    // instantiation of the component once in order to obtain "Model"
-    // property of the component
-    QString Name, Model;
-    char *File;
-    Component *c = (Component *) info(Name, File, true);
-    Model = c->Model;
+  // instantiation of the component once in order to obtain "Model"
+  // property of the component
+  QString Name;
+  char* File;
+  Component* c = (Component*)info(Name, File, true);
 
-    m->icon = new QPixmap(128,128);
+  // put into category and the component hash
+  if ((c->Simulator & QucsSettings.DefaultSimulator) ==
+      QucsSettings.DefaultSimulator) {
+    Module* m     = new Module();
+    m->info       = info;
+    m->category   = category;
+
+    m->icon = new QPixmap(128, 128);
     c->paintIcon(m->icon);
 
-    // put into category and the component hash
-    if ((c->Simulator & QucsSettings.DefaultSimulator) == QucsSettings.DefaultSimulator) {
-        intoCategory(m);
+    intoCategory(m);
+    if (!Modules.contains(c->Model)) {
+      Modules.insert(c->Model, m);
     }
-    delete c;
-    if (!Modules.contains(Model))
-        Modules.insert(Model, m);
+  }
+  delete c;
 }
 
 // Returns instantiated component based on the given "Model" name.  If
@@ -349,6 +351,8 @@ void Module::registerModules (void) {
   REGISTER_PROBE_1 (vProbe);
 
   // RF devices except transmission line
+  REGISTER_RF_COMP_1 (IndQ);
+  REGISTER_RF_COMP_1 (CapQ);
   REGISTER_RF_COMP_1 (dcBlock);
   REGISTER_RF_COMP_1 (dcFeed);
   REGISTER_RF_COMP_1 (circularloop);
@@ -410,14 +414,6 @@ void Module::registerModules (void) {
       REGISTER_NONLINEAR_3 (MOSFET, info, info_p, info_depl);
       REGISTER_NONLINEAR_3 (MOSFET_sub, info, info_p, info_depl);
   //} else {
-      REGISTER_NONLINEAR_1 (DIODE_SPICE);
-      REGISTER_NONLINEAR_1 (NPN_SPICE);
-      REGISTER_NONLINEAR_1 (PNP_SPICE);
-      REGISTER_NONLINEAR_2 (MOS_SPICE,info_NM3pin,info_PM3pin);
-      REGISTER_NONLINEAR_1 (NJF_SPICE);
-      REGISTER_NONLINEAR_1 (PJF_SPICE);
-      REGISTER_NONLINEAR_1 (NMOS_SPICE);
-      REGISTER_NONLINEAR_1 (PMOS_SPICE);
       REGISTER_NONLINEAR_1 (MESFET_SPICE);
       REGISTER_NONLINEAR_1 (PMF_MESFET_SPICE);
       REGISTER_NONLINEAR_1 (S4Q_Ieqndef);
@@ -426,6 +422,7 @@ void Module::registerModules (void) {
 
   REGISTER_NONLINEAR_1 (OpAmp);
   REGISTER_NONLINEAR_1 (EqnDefined);
+  REGISTER_NONLINEAR_1 (vcresistor);
 
   //if (QucsSettings.DefaultSimulator == spicecompat::simQucsator) {
       REGISTER_NONLINEAR_1 (Diac);
@@ -437,12 +434,17 @@ void Module::registerModules (void) {
 // PDK devices
       REGISTER_MICROEL_1 (R_SPICE, info_R3);
       REGISTER_MICROEL_1 (C_SPICE, info_C3);
+      REGISTER_MICROEL_1 (DIODE_SPICE, info);
       REGISTER_MICROEL_1 (DIODE_SPICE, info_DIODE3);
       REGISTER_MICROEL_1 (NMOS_SPICE, info);
       REGISTER_MICROEL_1 (PMOS_SPICE, info);
       REGISTER_MICROEL_2 (MOS_SPICE,info_NM3pin,info_PM3pin);
       REGISTER_MICROEL_2 (MOS_SPICE,info_NX3pin,info_PX3pin);
       REGISTER_MICROEL_2 (MOS_SPICE,info_NX4pin,info_PX4pin);
+      REGISTER_MICROEL_1 (NJF_SPICE, info);
+      REGISTER_MICROEL_1 (PJF_SPICE, info);
+      REGISTER_MICROEL_1 (NPN_SPICE, info);
+      REGISTER_MICROEL_1 (PNP_SPICE, info);
       REGISTER_MICROEL_2 (BJT_SPICE,infoNPN4,infoPNP4);
       REGISTER_MICROEL_2 (BJT_SPICE,infoNPN5,infoPNP5);
 
@@ -454,7 +456,6 @@ void Module::registerModules (void) {
       REGISTER_VERILOGA_1 (photodiode);
       REGISTER_VERILOGA_1 (phototransistor);
       REGISTER_VERILOGA_1 (nigbt);
-      REGISTER_VERILOGA_1 (vcresistor);
   //}
 
   // digital components
@@ -504,21 +505,12 @@ void Module::registerModules (void) {
   REGISTER_DIGITAL_1 (Verilog_File);
 
   // file components
-  REGISTER_FILE_1 (SpiceFile);
-  //if (QucsSettings.DefaultSimulator == spicecompat::simQucsator) {
-      REGISTER_FILE_3 (SParamFile, info1, info2, info);
-  //}
   REGISTER_FILE_1 (Subcircuit);
-  //if (QucsSettings.DefaultSimulator != spicecompat::simQucsator) {
-      REGISTER_FILE_1 (SpiceGeneric);
-      REGISTER_FILE_1 (SpiceLibComp);
-  //}
-
-  //if ((QucsSettings.DefaultSimulator == spicecompat::simNgspice)|| (QucsSettings.DefaultSimulator == spicecompat::simSpiceOpus)) {
-      REGISTER_FILE_1 (XspiceGeneric);
-      //REGISTER_FILE_1 (XSP_CMlib);
-      //REGISTER_FILE_1 (XSP_CodeModel);
-  //}
+  REGISTER_FILE_1 (SpiceLibComp);
+  REGISTER_FILE_1 (SpiceFile);
+  REGISTER_FILE_3 (SParamFile, info1, info2, info);
+  REGISTER_FILE_1 (SpiceGeneric);
+  REGISTER_FILE_1 (XspiceGeneric);
 
   // simulations
   REGISTER_SIMULATION_1 (DC_Sim);
@@ -565,6 +557,7 @@ void Module::registerModules (void) {
   // equations
   REGISTER_EQUATION_1 (NutmegEquation);
   REGISTER_EQUATION_1 (SpiceParam);
+  REGISTER_EQUATION_1 (SpiceCSParam);
   REGISTER_EQUATION_1 (SpiceGlobalParam);
   REGISTER_EQUATION_1 (Equation);
 
@@ -588,6 +581,7 @@ void Module::registerModules (void) {
 
       // specific sections of spice netlists
       REGISTER_SPICE_SEC_1 (SpiceParam);
+      REGISTER_SPICE_SEC_1 (SpiceCSParam);
       REGISTER_SPICE_SEC_1 (SpiceGlobalParam);
       REGISTER_SPICE_SEC_1 (SpiceOptions);
       REGISTER_SPICE_SEC_1 (SpiceIC);
@@ -625,27 +619,21 @@ void Module::registerModules (void) {
 
 // This function has to be called once at application end.  It removes
 // all categories and registered modules from memory.
-void Module::unregisterModules (void) {
-  while(!Category::Categories.isEmpty()) {
+void Module::unregisterModules(void) {
+  while (!Category::Categories.isEmpty()) {
     delete Category::Categories.takeFirst();
   }
 
-    QHash<QString, Module *>::iterator i = Modules.begin();
-    while (i != Modules.end()) {
-        if (i.value() == 0) {         // test here
-            i = Modules.erase(i);
-        } else {
-            ++i;
-        }
+  QHash<QString, Module*>::iterator i = Modules.begin();
+  while (i != Modules.end()) {
+    if (i.value() == 0) { // test here
+      delete i.value();
+      i = Modules.erase(i);
+    } else {
+      ++i;
     }
-
-//remove all modules by iterator, require in qhash
-//  QHashIterator<QString, Module *> it( Modules );
-//  while(it.hasNext()) {
-//    it.next();
-//    delete it.value();
-//  }
-  Modules.clear ();
+  }
+  Modules.clear();
 }
 
 // Constructor creates instance of module object.
