@@ -31,6 +31,7 @@
 #include "dialogs/textboxdialog.h"
 #include "dialogs/tuner.h"
 #include "extsimkernels/customsimdialog.h"
+#include "extsimkernels/spicelibcompdialog.h"
 #include "main.h"
 #include "module.h"
 #include "node.h"
@@ -60,7 +61,9 @@ QAction *formerAction; // remember action before drag n'drop etc.
 MouseActions::MouseActions(QucsApp *App_)
 {
     App = App_;          // pointer to main app
-    selElem = nullptr;         // no component/diagram is selected
+    if(selElem != nullptr){
+      selElem = nullptr;         // no component/diagram is selected
+    }
     isMoveEqual = false; // mouse cursor move x and y the same way
     focusElement = 0;    //element being interacted with mouse
 
@@ -68,7 +71,8 @@ MouseActions::MouseActions(QucsApp *App_)
     // initialize menu appearing by right mouse button click on component
     ComponentMenu = new QMenu(QucsMain);
     focusMEvent = new QMouseEvent(QEvent::MouseButtonPress,
-                                  QPoint(0, 0),
+                                  QPointF(0, 0),
+                                  QPointF(0, 0),
                                   Qt::NoButton,
                                   Qt::NoButton,
                                   Qt::NoModifier);
@@ -86,9 +90,8 @@ void MouseActions::setPainter(Schematic *Doc)
     // contents to viewport transformation
 
     Doc->PostPaintEvent(_Translate, -Doc->contentsX(), -Doc->contentsY());
-    Doc->PostPaintEvent(_Scale, Doc->Scale, Doc->Scale);
-    Doc->PostPaintEvent(_Translate, -Doc->ViewX1, -Doc->ViewY1);
-    Doc->PostPaintEvent(_DotLine);
+    Doc->PostPaintEvent(_Scale, Doc->getScale(), Doc->getScale());
+    Doc->PostPaintEvent(_Translate, -Doc->getViewX1(), -Doc->getViewY1());
     Doc->PostPaintEvent(_NotRop);
 }
 
@@ -102,12 +105,11 @@ bool MouseActions::pasteElements(Schematic *Doc)
     if (!Doc->paste(&stream, &movingElements))
         return false;
 
-    Element *pe;
     int xmax, xmin, ymax, ymin;
     xmin = ymin = INT_MAX;
     xmax = ymax = INT_MIN;
     // First, get the max and min coordinates of all selected elements.
-    for (pe = movingElements.first(); pe != 0; pe = movingElements.next()) {
+    for (auto* pe : movingElements) {
         if (pe->Type == isWire) {
             if (pe->x1 < xmin)
                 xmin = pe->x1;
@@ -134,7 +136,7 @@ bool MouseActions::pasteElements(Schematic *Doc)
     Doc->setOnGrid(xmin, ymin);
 
     // moving with mouse cursor in the midpoint
-    for (pe = movingElements.first(); pe != 0; pe = movingElements.next())
+    for (auto* pe : movingElements)
         if (pe->Type & isLabel) {
             pe->cx += xmin;
             pe->x1 += xmin;
@@ -176,18 +178,16 @@ void MouseActions::editLabel(Schematic *Doc, WireLabel *pl)
             pl->x1 -= pl->x2 - old_x2; // don't change position due to text width
     }
 
-    Doc->sizeOfAll(Doc->UsedX1, Doc->UsedY1, Doc->UsedX2, Doc->UsedY2);
+    Doc->updateAllBoundingRect();
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
 // -----------------------------------------------------------
 // Reinserts all elements (moved by the user) back into the schematic.
 void MouseActions::endElementMoving(Schematic *Doc,
-                                    Q3PtrList<Element> *movElements) {
-  Element *pe;
-  for (pe = movElements->first(); pe != nullptr; pe = movElements->next()) {
+                                    QList<Element*> *movElements) {
+  for (auto* pe : *movElements) {
     //    pe->isSelected = false;  // deselect first (maybe afterwards pe ==
     //    NULL)
     switch (pe->Type) { // FIXME: use casts.
@@ -206,10 +206,10 @@ void MouseActions::endElementMoving(Schematic *Doc,
       Doc->insertWire((Wire *)pe);
       break;
     case isDiagram:
-      Doc->Diagrams->append((Diagram *)pe);
+      Doc->a_Diagrams->append((Diagram *)pe);
       break;
     case isPainting:
-      Doc->Paintings->append((Painting *)pe);
+      Doc->a_Paintings->append((Painting *)pe);
       break;
     case isComponent:
     case isAnalogComponent:
@@ -232,21 +232,21 @@ void MouseActions::endElementMoving(Schematic *Doc,
   if ((MAx3 != 0) || (MAy3 != 0)) // moved or put at the same place ?
     Doc->setChanged(true, true);
 
-  // enlarge viewarea if components lie outside the view
-  Doc->sizeOfAll(Doc->UsedX1, Doc->UsedY1, Doc->UsedX2, Doc->UsedY2);
-  Doc->enlargeView(Doc->UsedX1, Doc->UsedY1, Doc->UsedX2, Doc->UsedY2);
+  // Position of some elements of schematic has changed. This change
+  // is "external" to schematic and its state must be updated to
+  // take the changes into account
+  auto totalBounds = Doc->allBoundingRect();
+  Doc->enlargeView(totalBounds.left(), totalBounds.top(), totalBounds.right(), totalBounds.bottom());
 
   Doc->viewport()->update();
-  drawn = false;
 }
 
 // -----------------------------------------------------------
 // Moves elements in "movElements" by x/y
-void MouseActions::moveElements(Q3PtrList<Element> *movElements, int x, int y)
+void MouseActions::moveElements(QList<Element*> *movElements, int x, int y)
 {
     Wire *pw;
-    Element *pe;
-    for (pe = movElements->first(); pe != 0; pe = movElements->next()) {
+    for (auto* pe : *movElements) {
         if (pe->Type == isWire) {
             pw = (Wire *) pe; // connected wires are not moved completely
 
@@ -320,17 +320,11 @@ void MouseActions::MMoveElement(Schematic *Doc, QMouseEvent *Event)
 
     if (selElem->Type == isPainting) {
         Doc->PostPaintEvent(_NotRop, 0, 0, 0, 0);
-        ((Painting *) selElem)->MouseMoving(Doc, x, y, gx, gy, Doc, fx, fy, drawn);
-        drawn = true;
+        ((Painting *) selElem)->MouseMoving(Doc, x, y, gx, gy, Doc, fx, fy);
         Doc->viewport()->update();
         return;
     } // of "if(isPainting)"
 
-    // ********** it is a component or diagram
-    if (drawn)
-        selElem->paintScheme(Doc); // erase old scheme
-    drawn = true;
-
     //  Component *comp = (Component*)selElem;
     //qDebug() << "desc" << comp->Description << "gx" << gx << "gy" << gy;
 
@@ -417,16 +411,7 @@ void MouseActions::MMoveWire2(Schematic *Doc, QMouseEvent *Event)
     Doc->setOnGrid(MAx2, MAy2);
     paintAim(Doc, MAx2, MAy2); //let we paint aim cross
 
-    //because cross slightly masks a wire, let we make wire thicker
-    //better to make it by increasing of pen, but here we cannot access
-    //pen
-    if (MAx1 == 0) {
-        paintGhostLineV(Doc, MAx3, MAy3, MAy2);
-        paintGhostLineH(Doc, MAx3, MAy2, MAx2);
-    } else {
-        paintGhostLineH(Doc, MAx3, MAy3, MAx2);
-        paintGhostLineV(Doc, MAx2, MAy3, MAy2);
-    }
+    Doc->showEphemeralWire({MAx3, MAy3}, {MAx2, MAy2});
 
     QucsMain->MouseDoubleClickAction = &MouseActions::MDoubleClickWire2;
     Doc->viewport()->update();
@@ -511,12 +496,21 @@ void MouseActions::MMoveMoving(Schematic *Doc, QMouseEvent *Event)
     MAy3 = MAy1 = MAy2 - MAy1;
 
     movingElements.clear();
-    Doc->copySelectedElements(&movingElements);
+    {
+        // Q3PtrList is long time deprecated and has to be replaced with another
+        // container type, which is not always easy. Here it's simpler to use it
+        // once and go back to QList, because copySelectedElements() uses API
+        // unique to Q3PtrList and to refactor it is a piece of work
+        Q3PtrList<Element> temp_buffer;
+        temp_buffer.setAutoDelete(false);
+        Doc->copySelectedElements(&temp_buffer);
+        for (auto* e : temp_buffer) { movingElements.append(e); }
+    }
     Doc->viewport()->repaint();
 
     Wire *pw;
     // Changes the position of all moving elements by dx/dy
-    for (Element *pe = movingElements.first(); pe != 0; pe = movingElements.next()) {
+    for (Element *pe : movingElements) {
         if (pe->Type == isWire) {
             pw = (Wire *) pe; // connecting wires are not moved completely
 
@@ -559,11 +553,13 @@ void MouseActions::MMoveMoving(Schematic *Doc, QMouseEvent *Event)
         pe->paintScheme(Doc);
     }
 
-    drawn = true;
     MAx1 = MAx2;
     MAy1 = MAy2;
     QucsMain->MouseMoveAction = &MouseActions::MMoveMoving2;
     QucsMain->MouseReleaseAction = &MouseActions::MReleaseMoving;
+    QucsMain->editRotate->blockSignals(true);
+    QucsMain->insLabel->blockSignals(true);
+    QucsMain->setMarker->blockSignals(true);
 }
 
 // -----------------------------------------------------------
@@ -576,12 +572,7 @@ void MouseActions::MMoveMoving2(Schematic *Doc, QMouseEvent *Event)
   MAx2 = inModel.x();
   MAy2 = inModel.y();
 
-  Element *pe;
-  if (drawn) // erase old scheme
-    for (pe = movingElements.first(); pe != nullptr; pe = movingElements.next())
-      pe->paintScheme(Doc);
 
-  drawn = true;
   if ((Event->modifiers().testFlag(Qt::ControlModifier)) == 0)
     Doc->setOnGrid(MAx2, MAy2); // use grid only if CTRL key not pressed
   MAx1 = MAx2 - MAx1;
@@ -592,7 +583,7 @@ void MouseActions::MMoveMoving2(Schematic *Doc, QMouseEvent *Event)
   moveElements(&movingElements, MAx1, MAy1); // moves elements by MAx1/MAy1
 
   // paint afterwards to avoid conflict between wire and label painting
-  for (pe = movingElements.first(); pe != 0; pe = movingElements.next())
+  for (auto* pe : movingElements)
     pe->paintScheme(Doc);
   //    if(pe->Type == isWire)  if(((Wire*)pe)->Label)
   //      if(!((Wire*)pe)->Label->isSelected)
@@ -615,7 +606,6 @@ void MouseActions::MMovePaste(Schematic *Doc, QMouseEvent *Event)
     moveElements(Doc, MAx1, MAy1);
     paintElementsScheme(Doc);
 
-    drawn = true;
     QucsMain->MouseMoveAction = &MouseActions::MMoveMoving2;
     QucsMain->MouseReleaseAction = &MouseActions::MReleasePaste;
 }
@@ -634,7 +624,7 @@ void MouseActions::MMoveScrollBar(Schematic *Doc, QMouseEvent *Event)
 
         // FIXME #warning QPainter p(Doc->viewport());
         // FIXME #warning ViewPainter Painter;
-        // FIXME #warning Painter.init(&p, Doc->Scale, -Doc->ViewX1, -Doc->ViewY1,
+        // FIXME #warning Painter.init(&p, Doc->a_Scale, -Doc->getViewX1(), -Doc->getViewY1(),
         // FIXME #warning                  Doc->contentsX(), Doc->contentsY());
         // FIXME #warning     Painter.fillRect(d->cx-d->x1, d->cy-d->y2, d->x2+d->x1, d->y2+d->y1,
         // FIXME #warning                      QucsSettings.BGColor);
@@ -911,7 +901,7 @@ void MouseActions::rightPressMenu(Schematic *Doc, QMouseEvent *Event, float fX,
                 Diagram* diagram = static_cast<Diagram*>(focusElement);
 
                 // Only show reset limits action if one or more axis is not autoscaled.
-                if (diagram->Name == "Rect" && 
+                if (diagram->Name == "Rect" &&
                     (!diagram->xAxis.autoScale || !diagram->yAxis.autoScale || !diagram->zAxis.autoScale)) {
                     ComponentMenu->addAction(QucsMain->resetDiagramLimits);
                 }
@@ -924,17 +914,6 @@ void MouseActions::rightPressMenu(Schematic *Doc, QMouseEvent *Event, float fX,
                                  SLOT(slotSaveDiagramToGraphicsFile()));
                 ComponentMenu->addAction(actExport);
             }
-            /*if (focusElement->Type & isComponent) {
-          Component *pc = (Component *)focusElement;
-          if (pc->Model == "EDD") {
-              QAction *actEDDtoIFS = new QAction(QObject::tr("Create XSPICE IFS"), QucsMain);
-              QObject::connect(actEDDtoIFS,SIGNAL(triggered(bool)),QucsMain,SLOT(slotEDDtoIFS()));
-              ComponentMenu->addAction(actEDDtoIFS);
-              QAction *actEDDtoMOD = new QAction(QObject::tr("Create XSPICE MOD"), QucsMain);
-              QObject::connect(actEDDtoMOD,SIGNAL(triggered(bool)),QucsMain,SLOT(slotEDDtoMOD()));
-              ComponentMenu->addAction(actEDDtoMOD);
-          }
-      }*/
         }
         break;
     }
@@ -1003,7 +982,6 @@ void MouseActions::rightPressMenu(Schematic *Doc, QMouseEvent *Event, float fX,
     ComponentMenu->popup(Event->globalPos());
 #endif
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -1075,9 +1053,8 @@ void MouseActions::MPressLabel(Schematic *Doc, QMouseEvent *, float fX, float fY
             pn->setName(Name, Value, xl, yl);
     }
 
-    Doc->sizeOfAll(Doc->UsedX1, Doc->UsedY1, Doc->UsedX2, Doc->UsedY2);
+    Doc->updateAllBoundingRect();
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -1099,7 +1076,7 @@ void MouseActions::MPressSelect(Schematic *Doc, QMouseEvent *Event, float fX, fl
     if (focusElement)
         // print define value in hex, see element.h
         qDebug() << "MPressSelect: focusElement->Type"
-                 << QString("0x%1").arg(focusElement->Type, 0, 16);
+                 << QStringLiteral("0x%1").arg(focusElement->Type, 0, 16);
     else
         qDebug() << "MPressSelect";
 
@@ -1175,7 +1152,6 @@ void MouseActions::MPressSelect(Schematic *Doc, QMouseEvent *Event, float fX, fl
             // Update matching wire label highlighting
             Doc->highlightWireLabels();
             Doc->viewport()->update();
-            drawn = false;
             return;
 
         case isComponentText: // property text of component ?
@@ -1217,7 +1193,6 @@ void MouseActions::MPressSelect(Schematic *Doc, QMouseEvent *Event, float fX, fl
     QucsMain->MouseDoubleClickAction = 0;
     Doc->grabKeyboard(); // no keyboard inputs during move actions
     Doc->viewport()->update();
-    drawn = false;
 
     if (focusElement == 0) {
         MAx2 = 0; // if not clicking on an element => open a rectangle
@@ -1246,9 +1221,8 @@ void MouseActions::MPressDelete(Schematic *Doc, QMouseEvent *, float fX, float f
         pe->isSelected = true;
         Doc->deleteElements();
 
-        Doc->sizeOfAll(Doc->UsedX1, Doc->UsedY1, Doc->UsedX2, Doc->UsedY2);
+        Doc->updateAllBoundingRect();
         Doc->viewport()->update();
-        drawn = false;
     }
 }
 
@@ -1266,7 +1240,6 @@ void MouseActions::MPressActivate(Schematic *Doc, QMouseEvent *, float fX, float
         QucsMain->MouseMoveAction = &MouseActions::MMoveSelect;
     }
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -1287,7 +1260,6 @@ void MouseActions::MPressMirrorX(Schematic *Doc, QMouseEvent *, float fX, float
     }
 
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -1309,7 +1281,6 @@ void MouseActions::MPressMirrorY(Schematic *Doc, QMouseEvent *, float fX, float
     }
 
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -1340,7 +1311,7 @@ void MouseActions::MPressRotate(Schematic *Doc, QMouseEvent *, float fX, float f
     case isWire:
         pl = ((Wire *) e)->Label;
         ((Wire *) e)->Label = 0; // prevent label to be deleted
-        Doc->Wires->setAutoDelete(false);
+        Doc->a_Wires->setAutoDelete(false);
         Doc->deleteWire((Wire *) e);
         ((Wire *) e)->Label = pl;
         ((Wire *) e)->rotate();
@@ -1349,8 +1320,8 @@ void MouseActions::MPressRotate(Schematic *Doc, QMouseEvent *, float fX, float f
         if (pl)
             Doc->setOnGrid(pl->cx, pl->cy);
         Doc->insertWire((Wire *) e);
-        Doc->Wires->setAutoDelete(true);
-        if (Doc->Wires->containsRef((Wire *) e))
+        Doc->a_Wires->setAutoDelete(true);
+        if (Doc->a_Wires->containsRef((Wire *) e))
             Doc->enlargeView(e->x1, e->y1, e->x2, e->y2);
         break;
 
@@ -1365,7 +1336,6 @@ void MouseActions::MPressRotate(Schematic *Doc, QMouseEvent *, float fX, float f
         return;
     }
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -1404,7 +1374,6 @@ void MouseActions::MPressElement(Schematic *Doc, QMouseEvent *Event, float, floa
             Doc->enlargeView(x1, y1, x2, y2);
             //Doc->setOnGrid(Comp->cx,Comp->cy);
 
-            drawn = false;
             Doc->viewport()->update();
             Doc->setChanged(true, true);
             rot = Comp->rotated;
@@ -1448,16 +1417,15 @@ void MouseActions::MPressElement(Schematic *Doc, QMouseEvent *Event, float, floa
             return;
 
         Diagram *Diag = (Diagram *) selElem;
-        QFileInfo Info(Doc->DocName);
+        QFileInfo Info(Doc->getDocName());
         // dialog is Qt::WDestructiveClose !!!
         DiagramDialog *dia = new DiagramDialog(Diag, Doc);
         if (dia->exec() == QDialog::Rejected) { // don't insert if dialog canceled
             Doc->viewport()->update();
-            drawn = false;
             return;
         }
 
-        Doc->Diagrams->append(Diag);
+        Doc->a_Diagrams->append(Diag);
         Doc->enlargeView(Diag->cx, Diag->cy - Diag->y2, Diag->cx + Diag->x2, Diag->cy);
         Doc->setChanged(true, true); // document has been changed
 
@@ -1470,7 +1438,7 @@ void MouseActions::MPressElement(Schematic *Doc, QMouseEvent *Event, float, floa
 
     // ***********  it is a painting !!!
     if (((Painting *) selElem)->MousePressing(Doc)) {
-        Doc->Paintings->append((Painting *) selElem);
+        Doc->a_Paintings->append((Painting *) selElem);
         ((Painting *) selElem)->Bounding(x1, y1, x2, y2);
         //Doc->enlargeView(x1, y1, x2, y2);
         selElem = ((Painting *) selElem)->newOne();
@@ -1479,7 +1447,6 @@ void MouseActions::MPressElement(Schematic *Doc, QMouseEvent *Event, float, floa
         Doc->setChanged(true, true);
 
         MMoveElement(Doc, Event); // needed before next mouse pressing
-        drawn = false;
     }
 }
 
@@ -1495,7 +1462,7 @@ void MouseActions::MPressWire1(Schematic *Doc, QMouseEvent *, float fX, float fY
     //Doc->PostPaintEvent (PPENotRop);
     //if(drawn) {
 #if 0 //ALYS - it draws some garbage, not deleted because of possible questions
-	Doc->PostPaintEvent (_Line, 0, MAy3, MAx2, MAy3); // erase old mouse cross
+    Doc->PostPaintEvent (_Line, 0, MAy3, MAx2, MAy3); // erase old mouse cross
     Doc->PostPaintEvent (_Line, MAx3, 0, MAx3, MAy2);
 #endif
     //}
@@ -1527,26 +1494,11 @@ void MouseActions::MPressWire1(Schematic *Doc, QMouseEvent *, float fX, float fY
  */
 void MouseActions::MPressWire2(Schematic *Doc, QMouseEvent *Event, float fX, float fY)
 {
-    int set1 = 0, set2 = 0;
     switch (Event->button()) {
-    case Qt::LeftButton:
-        if (MAx1 == 0) { // which wire direction first ?
-            if (MAy2 != MAy3)
-                set1 = Doc->insertWire(new Wire(MAx3, MAy3, MAx3, MAy2));
-            if (MAx2 != MAx3) {
-                set2 = set1;
-                set1 = Doc->insertWire(new Wire(MAx3, MAy2, MAx2, MAy2));
-            }
-        } else {
-            if (MAx2 != MAx3)
-                set1 = Doc->insertWire(new Wire(MAx3, MAy3, MAx2, MAy3));
-            if (MAy2 != MAy3) {
-                set2 = set1;
-                set1 = Doc->insertWire(new Wire(MAx2, MAy3, MAx2, MAy2));
-            }
-        }
+    case Qt::LeftButton: {
+        auto [hasChanges, lastNode] = Doc->connectWithWire({MAx3, MAy3}, {MAx2, MAy2});
 
-        if (set1 & 2) {
+        if (lastNode == nullptr || lastNode->conn_count() > 1) {
             // if last port is connected, then...
             if (formerAction) {
                 // ...restore old action
@@ -1559,24 +1511,20 @@ void MouseActions::MPressWire2(Schematic *Doc, QMouseEvent *Event, float fX, flo
             }
         }
 
-        //ALYS: excessive update. end of function does it.
-        //Doc->viewport()->update();
-
-        drawn = false;
-        if (set1 | set2)
+        if (hasChanges)
             Doc->setChanged(true, true);
         MAx3 = MAx2;
         MAy3 = MAy2;
         break;
-
+    }
         /// \todo document right mouse button changes the wire corner
     case Qt::RightButton:
-
+        Doc->a_wirePlanner.next();
 #if 0
-	//ALYS - old code preserved because isn't clear - what it was???
-	//looks like deletion via painting.
-	//i'll delete it after possible clarification from team
-	if(MAx1 == 0) {
+    //ALYS - old code preserved because isn't clear - what it was???
+    //looks like deletion via painting.
+    //i'll delete it after possible clarification from team
+    if(MAx1 == 0) {
       Doc->PostPaintEvent (_Line, MAx3, MAy3, MAx3, MAy2); // erase old
       Doc->PostPaintEvent (_Line, MAx3, MAy2, MAx2, MAy2); // erase old
     }
@@ -1591,13 +1539,7 @@ void MouseActions::MPressWire2(Schematic *Doc, QMouseEvent *Event, float fX, flo
         Doc->setOnGrid(MAx2, MAy2);
 
         MAx1 ^= 1; // change the painting direction of wire corner
-        if (MAx1 == 0) {
-            paintGhostLineV(Doc, MAx3, MAy3, MAy2);
-            paintGhostLineH(Doc, MAx3, MAy2, MAx2);
-        } else {
-            paintGhostLineH(Doc, MAx3, MAy3, MAx2);
-            paintGhostLineV(Doc, MAx2, MAy3, MAy2);
-        }
+        Doc->showEphemeralWire({MAx3, MAy3}, {MAx2, MAy2});
         break;
 
     default:; // avoids compiler warnings
@@ -1622,7 +1564,6 @@ void MouseActions::MPressMarker(Schematic *Doc, QMouseEvent *, float fX, float f
         Doc->enlargeView(x0 + pm->x1, y0 - pm->y1 - pm->y2, x0 + pm->x1 + pm->x2, y0 - pm->y1);
     }
     Doc->viewport()->update();
-    drawn = false;
 }
 
 /**
@@ -1640,7 +1581,7 @@ void MouseActions::MPressSetLimits(Schematic *Doc, QMouseEvent*, float fX, float
     // TODO: Diagrams is currently a Q3PtrList, but it would be better to refactor
     // this (and many other collections) to be std::vector.
     // Check to see if the mouse is within a diagram using the oddly named "getSelected".
-    for (Diagram* diagram = Doc->Diagrams->last(); diagram != 0; diagram = Doc->Diagrams->prev()) {
+    for (Diagram* diagram = Doc->a_Diagrams->last(); diagram != 0; diagram = Doc->a_Diagrams->prev()) {
         // BUG: Obtaining the diagram type by name is marked as a bug elsewhere (to be solved separately).
         // TODO: Currently only rectangular diagrams are supported.
         if (diagram->getSelected(fX, fY) && diagram->Name == "Rect") {
@@ -1653,14 +1594,13 @@ void MouseActions::MPressSetLimits(Schematic *Doc, QMouseEvent*, float fX, float
             QucsMain->MouseMoveAction = &MouseActions::MMoveSelect;
             QucsMain->MouseReleaseAction = &MouseActions::MReleaseSetLimits;
             Doc->grabKeyboard(); // no keyboard inputs during move actions
-            
+
             // No need to continue searching;
             break;
         }
     }
 
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -1674,11 +1614,10 @@ void MouseActions::MPressOnGrid(Schematic *Doc, QMouseEvent *, float fX, float f
         pe->isSelected = true;
         Doc->elementsOnGrid();
 
-        Doc->sizeOfAll(Doc->UsedX1, Doc->UsedY1, Doc->UsedX2, Doc->UsedY2);
+        Doc->updateAllBoundingRect();
         // Update matching wire label highlighting
         Doc->highlightWireLabels();
         Doc->viewport()->update();
-        drawn = false;
     }
 }
 
@@ -1695,7 +1634,6 @@ void MouseActions::MPressMoveText(Schematic *Doc, QMouseEvent *, float fX, float
         MAx1 = ((Component *) focusElement)->cx + ((Component *) focusElement)->tx;
         MAy1 = ((Component *) focusElement)->cy + ((Component *) focusElement)->ty;
         Doc->viewport()->update();
-        drawn = false;
         QucsMain->MouseMoveAction = &MouseActions::MMoveMoveText;
         QucsMain->MouseReleaseAction = &MouseActions::MReleaseMoveText;
         Doc->grabKeyboard(); // no keyboard inputs during move actions
@@ -1715,7 +1653,6 @@ void MouseActions::MPressZoomIn(Schematic *Doc, QMouseEvent *, float fX, float f
     QucsMain->MouseReleaseAction = &MouseActions::MReleaseZoomIn;
     Doc->grabKeyboard(); // no keyboard inputs during move actions
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // ***********************************************************************
@@ -1748,7 +1685,6 @@ void MouseActions::MReleaseSelect(Schematic *Doc, QMouseEvent *Event)
     QucsMain->MouseMoveAction = 0; // no element moving
     Doc->highlightWireLabels();
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -1762,7 +1698,8 @@ void MouseActions::MReleaseSelect2(Schematic *Doc, QMouseEvent *Event)
     bool IsShift = Event->modifiers().testFlag(Qt::ShiftModifier);
 
     // selects all elements within the rectangle
-    Doc->selectElements(MAx1, MAy1, MAx1 + MAx2, MAy1 + MAy2, IsCtrl, !IsShift);
+    Doc->selectElements(
+        QRect{MAx1, MAy1, MAx2, MAy2}.normalized(), IsCtrl, !IsShift);
 
     Doc->releaseKeyboard(); // allow keyboard inputs again
     QucsMain->MouseMoveAction = 0;
@@ -1770,9 +1707,8 @@ void MouseActions::MReleaseSelect2(Schematic *Doc, QMouseEvent *Event)
     QucsMain->MouseReleaseAction = &MouseActions::MReleaseSelect;
     QucsMain->MouseDoubleClickAction = &MouseActions::MDoubleClickSelect;
     Doc->highlightWireLabels();
-    Doc->PostedPaintEvents.clear();
+    Doc->clearPostedPaintEvents();
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -1790,7 +1726,6 @@ void MouseActions::MReleaseActivate(Schematic *Doc, QMouseEvent *Event)
     QucsMain->MouseDoubleClickAction = 0;
     Doc->highlightWireLabels();
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -1806,6 +1741,10 @@ void MouseActions::MReleaseMoving(Schematic *Doc, QMouseEvent *)
     QucsMain->MousePressAction = &MouseActions::MPressSelect;
     QucsMain->MouseReleaseAction = &MouseActions::MReleaseSelect;
     QucsMain->MouseDoubleClickAction = &MouseActions::MDoubleClickSelect;
+    QucsMain->editRotate->setChecked(false);
+    QucsMain->editRotate->blockSignals(false);
+    QucsMain->insLabel->blockSignals(false);
+    QucsMain->setMarker->blockSignals(false);
 }
 
 // -----------------------------------------------------------
@@ -1860,7 +1799,6 @@ void MouseActions::MReleaseResizeDiagram(Schematic *Doc, QMouseEvent *Event)
     Doc->releaseKeyboard(); // allow keyboard inputs again
 
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -1877,7 +1815,6 @@ void MouseActions::MReleaseResizePainting(Schematic *Doc, QMouseEvent *Event)
     Doc->releaseKeyboard(); // allow keyboard inputs again
 
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -1893,16 +1830,16 @@ void MouseActions::MReleaseSetLimits(Schematic *Doc, QMouseEvent *Event)
 
     qDebug() << "Mouse released after setting limits.";
     // Check to see if the mouse is within a diagram using the oddly named "getSelected".
-    for (Diagram* diagram = Doc->Diagrams->last(); diagram != 0; diagram = Doc->Diagrams->prev()) {
-        
+    for (Diagram* diagram = Doc->a_Diagrams->last(); diagram != 0; diagram = Doc->a_Diagrams->prev()) {
+
         // Only process the selection if it ends in a diagram, and is the same diagram as start.
         if (diagram->getSelected(MAx2, MAy2) && diagram == pActiveDiagram) {
             qDebug() << "In the same diagram, setting limits";
-            
+
             mouseUpPoint = QPointF(MAx2 - diagram->cx, diagram->cy - MAy2);
 
             // Normalise the selection in case user starts at bottom and/or right.
-            QRectF select = QRectF(mouseDownPoint, mouseUpPoint).normalized();      
+            QRectF select = QRectF(mouseDownPoint, mouseUpPoint).normalized();
 
             // Only process if there is a valid selection box.
             if (select.width() < MIN_SELECT_SIZE || select.height() < MIN_SELECT_SIZE)
@@ -1910,10 +1847,7 @@ void MouseActions::MReleaseSetLimits(Schematic *Doc, QMouseEvent *Event)
 
             diagram->setLimitsBySelectionRect(select);
 
-            // TODO: Consider refactoring loadGraphData to reload the current dataset if an empty string is passed.
-            QFileInfo Info(Doc->DocName);
-            QString defaultDataSet = Info.absolutePath() + QDir::separator() + Doc->DataSet;
-            diagram->loadGraphData(defaultDataSet);
+            diagram->updateGraphData();
 
             Doc->setChanged(true, true);
 
@@ -1931,18 +1865,16 @@ void MouseActions::MReleaseSetLimits(Schematic *Doc, QMouseEvent *Event)
 // -----------------------------------------------------------
 void MouseActions::paintElementsScheme(Schematic *p)
 {
-    Element *pe;
-    for (pe = movingElements.first(); pe != nullptr; pe = movingElements.next())
+    for (auto* pe : movingElements)
         pe->paintScheme(p);
 }
 
 // -----------------------------------------------------------
 void MouseActions::moveElements(Schematic *Doc, int &x1, int &y1)
 {
-    Element *pe;
     Doc->setOnGrid(x1, y1);
 
-    for (pe = movingElements.first(); pe != 0; pe = movingElements.next()) {
+    for (auto* pe : movingElements) {
         if (pe->Type & isLabel) {
             pe->cx += x1;
             pe->x1 += x1;
@@ -1957,10 +1889,9 @@ void MouseActions::moveElements(Schematic *Doc, int &x1, int &y1)
 void MouseActions::rotateElements(Schematic *Doc, int &x1, int &y1)
 {
     int x2;
-    Element *pe;
     Doc->setOnGrid(x1, y1);
 
-    for (pe = movingElements.first(); pe != 0; pe = movingElements.next()) {
+    for (auto* pe : movingElements) {
         switch (pe->Type) {
         case isComponent:
         case isAnalogComponent:
@@ -1992,14 +1923,13 @@ void MouseActions::rotateElements(Schematic *Doc, int &x1, int &y1)
 void MouseActions::MReleasePaste(Schematic *Doc, QMouseEvent *Event)
 {
     int x1, y1, x2, y2, rot;
-    QFileInfo Info(Doc->DocName);
+    QFileInfo Info(Doc->getDocName());
     //QPainter painter(Doc->viewport());
 
-    Element *pe;
     switch (Event->button()) {
     case Qt::LeftButton:
         // insert all moved elements into document
-        for (pe = movingElements.first(); pe != 0; pe = movingElements.next()) {
+        for (auto* pe : movingElements) {
             pe->isSelected = false;
             switch (pe->Type) {
             case isWire:
@@ -2007,19 +1937,19 @@ void MouseActions::MReleasePaste(Schematic *Doc, QMouseEvent *Event)
                     if (pe->y1 == pe->y2)
                         break;
                 Doc->insertWire((Wire *) pe);
-                if (Doc->Wires->containsRef((Wire *) pe))
+                if (Doc->a_Wires->containsRef((Wire *) pe))
                     Doc->enlargeView(pe->x1, pe->y1, pe->x2, pe->y2);
                 else
                     pe = NULL;
                 break;
             case isDiagram:
-                Doc->Diagrams->append((Diagram *) pe);
+                Doc->a_Diagrams->append((Diagram *) pe);
                 ((Diagram *) pe)
-                    ->loadGraphData(Info.absolutePath() + QDir::separator() + Doc->DataSet);
+                    ->loadGraphData(Info.absolutePath() + QDir::separator() + Doc->getDataSet());
                 Doc->enlargeView(pe->cx, pe->cy - pe->y2, pe->cx + pe->x2, pe->cy);
                 break;
             case isPainting:
-                Doc->Paintings->append((Painting *) pe);
+                Doc->a_Paintings->append((Painting *) pe);
                 ((Painting *) pe)->Bounding(x1, y1, x2, y2);
                 Doc->enlargeView(x1, y1, x2, y2);
                 break;
@@ -2049,7 +1979,6 @@ void MouseActions::MReleasePaste(Schematic *Doc, QMouseEvent *Event)
         QucsMain->MouseReleaseAction = 0;
         QucsMain->MouseDoubleClickAction = 0;
 
-        drawn = false;
         Doc->viewport()->update();
         Doc->setChanged(true, true);
         break;
@@ -2058,10 +1987,6 @@ void MouseActions::MReleasePaste(Schematic *Doc, QMouseEvent *Event)
     case Qt::RightButton: {// right button rotates the elements
         //setPainter(Doc, &painter);
 
-        if (drawn) // erase old scheme
-            paintElementsScheme(Doc);
-        drawn = true;
-
         auto inModel = Doc->contentsToModel(Event->pos());
         x1 = inModel.x();
         y1 = inModel.y();
@@ -2090,7 +2015,6 @@ void MouseActions::MReleaseMoveText(Schematic *Doc, QMouseEvent *Event)
     ((Component *) focusElement)->tx = MAx1 - ((Component *) focusElement)->cx;
     ((Component *) focusElement)->ty = MAy1 - ((Component *) focusElement)->cy;
     Doc->viewport()->update();
-    drawn = false;
     Doc->setChanged(true, true);
 }
 
@@ -2134,7 +2058,7 @@ void MouseActions::editElement(Schematic *Doc, QMouseEvent *Event)
     MarkerDialog *mdia;
     int x1, y1, x2, y2;
 
-    QFileInfo Info(Doc->DocName);
+    QFileInfo Info(Doc->getDocName());
     auto inModel = Doc->contentsToModel(Event->pos());
     float fX = static_cast<float>(inModel.x());
     float fY = static_cast<float>(inModel.y());
@@ -2148,7 +2072,12 @@ void MouseActions::editElement(Schematic *Doc, QMouseEvent *Event)
         if (c->Model == "GND")
             return;
 
-        if ((c->Model == ".CUSTOMSIM") || (c->Model == ".XYCESCR") || (c->Model == "INCLSCR")) {
+        if (c->Model == "SpLib") {
+          SpiceLibCompDialog *sld = new SpiceLibCompDialog(c, Doc);
+          if (sld->exec() != -1) {
+            break;
+          }
+        } else if ((c->Model == ".CUSTOMSIM") || (c->Model == ".XYCESCR") || (c->Model == "INCLSCR")) {
             CustomSimDialog *sd = new CustomSimDialog((SpiceCustomSim *) c, Doc);
             if (sd->exec() != 1)
                 break; // dialog is WDestructiveClose
@@ -2169,10 +2098,10 @@ void MouseActions::editElement(Schematic *Doc, QMouseEvent *Event)
             if (cd->exec() != 1)
                 break; // dialog is WDestructiveClose
 
-            Doc->Components->findRef(c);
-            Doc->Components->take();
+            Doc->a_Components->findRef(c);
+            Doc->a_Components->take();
             Doc->setComponentNumber(c); // for ports/power sources
-            Doc->Components->append(c);
+            Doc->a_Components->append(c);
         }
 
         Doc->setChanged(true, true);
@@ -2182,7 +2111,8 @@ void MouseActions::editElement(Schematic *Doc, QMouseEvent *Event)
 
     case isDiagram:
         dia = (Diagram *) focusElement;
-        if (dia->Name.at(0) == 'T') { // don't open dialog on scrollbar
+        if (dia->Name.at(0) == 'T' // check only on double click
+            && Event->type() == QMouseEvent::MouseButtonDblClick) { // don't open dialog on scrollbar
             if (dia->Name == "Time") {
                 if (dia->cy < int(fY)) {
                     if (((TimingDiagram *) focusElement)->scroll(MAx1))
@@ -2209,7 +2139,7 @@ void MouseActions::editElement(Schematic *Doc, QMouseEvent *Event)
     case isGraph:
         pg = (Graph *) focusElement;
         // searching diagram for this graph
-        for (dia = Doc->Diagrams->last(); dia != 0; dia = Doc->Diagrams->prev())
+        for (dia = Doc->a_Diagrams->last(); dia != 0; dia = Doc->a_Diagrams->prev())
             if (dia->Graphs.indexOf(pg) >= 0)
                 break;
         if (!dia)
@@ -2251,7 +2181,6 @@ void MouseActions::editElement(Schematic *Doc, QMouseEvent *Event)
     Doc->setFocus();
 
     Doc->viewport()->update();
-    drawn = false;
 }
 
 // -----------------------------------------------------------
@@ -2293,7 +2222,7 @@ void MouseActions::MPressTune(Schematic *Doc, QMouseEvent *Event, float fX, floa
     if (focusElement)
         // print define value in hex, see element.h
         qDebug() << "MPressTune: focusElement->Type"
-                 << QString("0x%1").arg(focusElement->Type, 0, 16);
+                 << QStringLiteral("0x%1").arg(focusElement->Type, 0, 16);
     else
         qDebug() << "MPressTune";
 
@@ -2333,7 +2262,7 @@ void MouseActions::MPressTune(Schematic *Doc, QMouseEvent *Event, float fX, floa
         if (!App->tunerDia->containsProperty(pp)) {
             if (isPropertyTunable(pc, pp)) {
                 tunerElement *tune = new tunerElement(App->tunerDia, pc, pp, No);
-                tune->schematicName = Doc->DocName;
+                tune->schematicName = Doc->getDocName();
                 if (tune != NULL)
                     App->tunerDia->addTunerElement(tune); //Tunable property
             } else {
