@@ -41,6 +41,7 @@
 #include <QListWidget>
 #include <QDesktopServices>
 
+#include "portsymbol.h"
 #include "projectView.h"
 #include "main.h"
 #include "qucs.h"
@@ -58,7 +59,6 @@
 #include "dialogs/librarydialog.h"
 #include "dialogs/loaddialog.h"
 #include "dialogs/importdialog.h"
-#include "dialogs/packagedialog.h"
 #include "dialogs/aboutdialog.h"
 #include "module.h"
 
@@ -75,7 +75,7 @@ QRegularExpressionValidator Val_CompProp(Expr_CompProp, 0);
 // -----------------------------------------------------------------------
 // This function is called from all toggle actions.
 bool QucsApp::performToggleAction(bool on, QAction *Action,
-	pToggleFunc Function, pMouseFunc MouseMove, pMouseFunc2 MousePress)
+      pToggleFunc Function, pMouseFunc MouseMove, pMouseFunc2 MousePress)
 {
   slotHideEdit(); // disable text edit of component property
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
@@ -117,7 +117,6 @@ bool QucsApp::performToggleAction(bool on, QAction *Action,
   } while(false);   // to perform "break"
 
   Doc->viewport()->update();
-  view->drawn = false;
   return true;
 }
 
@@ -126,7 +125,7 @@ bool QucsApp::performToggleAction(bool on, QAction *Action,
 void QucsApp::slotOnGrid(bool on)
 {
   performToggleAction(on, onGrid, &Schematic::elementsOnGrid,
-		&MouseActions::MMoveOnGrid, &MouseActions::MPressOnGrid);
+    &MouseActions::MMoveOnGrid, &MouseActions::MPressOnGrid);
 }
 
 // -----------------------------------------------------------------------
@@ -134,7 +133,7 @@ void QucsApp::slotOnGrid(bool on)
 void QucsApp::slotEditRotate(bool on)
 {
   performToggleAction(on, editRotate, &Schematic::rotateElements,
-		&MouseActions::MMoveRotate, &MouseActions::MPressRotate);
+    &MouseActions::MMoveRotate, &MouseActions::MPressRotate);
 }
 
 // -----------------------------------------------------------------------
@@ -142,7 +141,7 @@ void QucsApp::slotEditRotate(bool on)
 void QucsApp::slotEditMirrorX(bool on)
 {
   performToggleAction(on, editMirror, &Schematic::mirrorXComponents,
-		&MouseActions::MMoveMirrorX, &MouseActions::MPressMirrorX);
+    &MouseActions::MMoveMirrorX, &MouseActions::MPressMirrorX);
 }
 
 // -----------------------------------------------------------------------
@@ -150,7 +149,7 @@ void QucsApp::slotEditMirrorX(bool on)
 void QucsApp::slotEditMirrorY(bool on)
 {
   performToggleAction(on, editMirrorY, &Schematic::mirrorYComponents,
-		&MouseActions::MMoveMirrorY, &MouseActions::MPressMirrorY);
+    &MouseActions::MMoveMirrorY, &MouseActions::MPressMirrorY);
 }
 
 // -----------------------------------------------------------------------
@@ -198,21 +197,21 @@ void QucsApp::slotEditDelete(bool on)
 void QucsApp::slotSetWire(bool on)
 {
   performToggleAction(on, insWire, 0,
-		&MouseActions::MMoveWire1, &MouseActions::MPressWire1);
+    &MouseActions::MMoveWire1, &MouseActions::MPressWire1);
 }
 
 // -----------------------------------------------------------------------
 void QucsApp::slotInsertLabel(bool on)
 {
   performToggleAction(on, insLabel, 0,
-		&MouseActions::MMoveLabel, &MouseActions::MPressLabel);
+    &MouseActions::MMoveLabel, &MouseActions::MPressLabel);
 }
 
 // -----------------------------------------------------------------------
 void QucsApp::slotSetMarker(bool on)
 {
   performToggleAction(on, setMarker, 0,
-		&MouseActions::MMoveMarker, &MouseActions::MPressMarker);
+    &MouseActions::MMoveMarker, &MouseActions::MPressMarker);
 }
 
 // -----------------------------------------------------------------------
@@ -220,7 +219,7 @@ void QucsApp::slotSetMarker(bool on)
 void QucsApp::slotSetDiagramLimits(bool on)
 {
   performToggleAction(on, setDiagramLimits, 0,
-	                    &MouseActions::MMoveSetLimits, &MouseActions::MPressSetLimits);
+                    &MouseActions::MMoveSetLimits, &MouseActions::MPressSetLimits);
 }
 
 // -----------------------------------------------------------------------
@@ -228,18 +227,15 @@ void QucsApp::slotSetDiagramLimits(bool on)
 void QucsApp::slotResetDiagramLimits()
 {
   if (view->focusElement && view->focusElement->Type == isDiagram)
-  {   
-    Diagram* diagram = (Diagram*)(view->focusElement);
-    
+  {
+    Diagram* diagram = dynamic_cast<Diagram*>(view->focusElement);
+    Schematic* Doc = dynamic_cast<Schematic*>(DocumentTab->currentWidget());
+
     diagram->xAxis.autoScale = true;
     diagram->yAxis.autoScale = true;
     diagram->zAxis.autoScale = true;
 
-    // Now read in the data.
-    auto* Doc = (Schematic*)DocumentTab->currentWidget();
-    QFileInfo Info(Doc->DocName);
-    QString defaultDataSet = Info.absolutePath() + QDir::separator() + Doc->DataSet;
-    diagram->loadGraphData(defaultDataSet);
+    diagram->updateGraphData();
 
     Doc->setChanged(true, true);
     Doc->viewport()->update();
@@ -249,12 +245,25 @@ void QucsApp::slotResetDiagramLimits()
   slotEscape();
 }
 
+// -----------------------------------------------------------------------
+// Is called, when "show grid" action is triggered.
+void QucsApp::slotShowGrid()
+{
+  qDebug() << "slotShowGrid";
+  Schematic* schematic = static_cast<Schematic*>(DocumentTab->currentWidget());
+  if(!isTextDocument(schematic)) {
+    schematic->setGridOn(!schematic->getGridOn());
+    schematic->setChanged(true);
+    schematic->viewport()->repaint();
+  }
+}
+
 // -----------------------------------------------------------------------
 // Is called, when "move component text" action is triggered.
 void QucsApp::slotMoveText(bool on)
 {
   performToggleAction(on, moveText, 0,
-		&MouseActions::MMoveMoveTextB, &MouseActions::MPressMoveText);
+    &MouseActions::MMoveMoveTextB, &MouseActions::MPressMoveText);
 }
 
 // -----------------------------------------------------------------------
@@ -270,7 +279,7 @@ void QucsApp::slotZoomIn(bool on)
   }
   else
     performToggleAction(on, magPlus, 0,
-		&MouseActions::MMoveZoomIn, &MouseActions::MPressZoomIn);
+      &MouseActions::MMoveZoomIn, &MouseActions::MPressZoomIn);
 }
 
 
@@ -299,7 +308,6 @@ void QucsApp::slotSelect(bool on)
     MouseMoveAction = &MouseActions::MMoveWire1;
     MousePressAction = &MouseActions::MPressWire1;
     Doc->viewport()->update();
-    view->drawn = false;
 
     select->blockSignals(true);
     select->setChecked(false);
@@ -373,9 +381,6 @@ void QucsApp::slotEditPaste(bool on)
       MouseReleaseAction = 0;
       MouseDoubleClickAction = 0;
       activeAction = 0;   // no action active
-      if(view->drawn) {
-        ((Schematic *)Doc)->viewport()->update();
-      }
       return;
     }
 
@@ -395,7 +400,6 @@ void QucsApp::slotEditPaste(bool on)
     }
     activeAction = editPaste;
 
-    view->drawn = false;
     MouseMoveAction = &MouseActions::MMovePaste;
     view->movingRotated = 0;
     MousePressAction = 0;
@@ -449,9 +453,6 @@ void QucsApp::slotInsertEquation(bool on)
       view->selElem = new Equation();
   }
 
-  Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
-  if(view->drawn) Doc->viewport()->update();
-  view->drawn = false;
   MouseMoveAction = &MouseActions::MMoveElement;
   MousePressAction = &MouseActions::MPressElement;
 }
@@ -482,9 +483,6 @@ void QucsApp::slotInsertGround(bool on)
 
   view->selElem = new Ground();
 
-  Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
-  if(view->drawn) Doc->viewport()->update();
-  view->drawn = false;
   MouseMoveAction = &MouseActions::MMoveElement;
   MousePressAction = &MouseActions::MPressElement;
 }
@@ -513,11 +511,13 @@ void QucsApp::slotInsertPort(bool on)
   if(view->selElem)
     delete view->selElem;  // delete previously selected component
 
-  view->selElem = new SubCirPort();
-
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
-  if(view->drawn) Doc->viewport()->update();
-  view->drawn = false;
+  if (Doc->getSymbolMode()) {
+    view->selElem = new PortSymbol();
+  } else {
+     view->selElem = new SubCirPort();
+  }
+
   MouseMoveAction = &MouseActions::MMoveElement;
   MousePressAction = &MouseActions::MPressElement;
 }
@@ -537,7 +537,6 @@ void QucsApp::slotEditUndo()
 
   Doc->undo();
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -555,7 +554,6 @@ void QucsApp::slotEditRedo()
 
   Doc->redo();
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -567,9 +565,8 @@ void QucsApp::slotAlignTop()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   if(!Doc->aligning(0))
     QMessageBox::information(this, tr("Info"),
-		      tr("At least two elements must be selected !"));
+          tr("At least two elements must be selected !"));
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -581,9 +578,8 @@ void QucsApp::slotAlignBottom()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   if(!Doc->aligning(1))
     QMessageBox::information(this, tr("Info"),
-		      tr("At least two elements must be selected !"));
+          tr("At least two elements must be selected !"));
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -595,9 +591,8 @@ void QucsApp::slotAlignLeft()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   if(!Doc->aligning(2))
     QMessageBox::information(this, tr("Info"),
-		      tr("At least two elements must be selected !"));
+          tr("At least two elements must be selected !"));
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -609,9 +604,8 @@ void QucsApp::slotAlignRight()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   if(!Doc->aligning(3))
     QMessageBox::information(this, tr("Info"),
-		      tr("At least two elements must be selected !"));
+          tr("At least two elements must be selected !"));
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -623,7 +617,6 @@ void QucsApp::slotDistribHoriz()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   Doc->distributeHorizontal();
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -635,7 +628,6 @@ void QucsApp::slotDistribVert()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   Doc->distributeVertical();
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -647,9 +639,8 @@ void QucsApp::slotCenterHorizontal()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   if(!Doc->aligning(4))
     QMessageBox::information(this, tr("Info"),
-		      tr("At least two elements must be selected !"));
+          tr("At least two elements must be selected !"));
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // --------------------------------------------------------------
@@ -661,9 +652,8 @@ void QucsApp::slotCenterVertical()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   if(!Doc->aligning(5))
     QMessageBox::information(this, tr("Info"),
-		      tr("At least two elements must be selected !"));
+          tr("At least two elements must be selected !"));
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 // ---------------------------------------------------------------------
@@ -679,15 +669,9 @@ void QucsApp::slotSelectAll()
     ((TextDoc*)Doc)->selectAll();
   }
   else {
-    int xmin, ymin, xmax, ymax;
-    ((Schematic*)Doc)->sizeOfAll(xmin, ymin, xmax, ymax);
-    xmin--;
-    ymin--;
-    xmax++;
-    ymax++;
-    ((Schematic*)Doc)->selectElements(xmin, ymin, xmax, ymax, true, false);
+    auto selectionRect = ((Schematic*)Doc)->allBoundingRect().marginsAdded(QMargins{1, 1, 1, 1});
+    ((Schematic*)Doc)->selectElements(selectionRect, true, false);
     ((Schematic*)Doc)->viewport()->update();
-    view->drawn = false;
   }
 }
 
@@ -700,7 +684,6 @@ void QucsApp::slotSelectMarker()
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
   Doc->selectMarkers();
   Doc->viewport()->update();
-  view->drawn = false;
 }
 
 
@@ -742,7 +725,7 @@ void QucsApp::editFile(const QString& File)
 
       if (QucsSettings.Editor.toLower().contains("qucsedit")) {
 
-#ifdef __MINGW32__
+#if defined(_WIN32) || defined(__MINGW32__)
   prog = QUCS_NAME"edit.exe";
 #elif __APPLE__
   prog = "qucsedit.app/Contents/MacOS/qucsedit";
@@ -787,7 +770,7 @@ void QucsApp::editFile(const QString& File)
 // Is called to show the output messages of the last simulation.
 void QucsApp::slotShowLastMsg()
 {
-  editFile(QucsSettings.QucsHomeDir.filePath("log.txt"));
+  editFile(QucsSettings.tempFilesDir.filePath("log.txt"));
 }
 
 // ------------------------------------------------------------------------
@@ -814,7 +797,7 @@ void QucsApp::slotShowLastNetlist()
 
     switch (QucsSettings.DefaultSimulator) {
     case spicecompat::simQucsator :
-        netlists.append(QucsSettings.QucsHomeDir.filePath("netlist.txt"));
+        netlists.append(QucsSettings.tempFilesDir.filePath("netlist.txt"));
         break;
     case spicecompat::simNgspice :
     case spicecompat::simSpiceOpus :
@@ -835,7 +818,7 @@ void QucsApp::slotShowLastNetlist()
         Schematic *sch = (Schematic *) w;
         if (sch->isDigitalCircuit()) {
             netlists.clear();
-            netlists.append(QucsSettings.QucsHomeDir.filePath("netlist.txt"));
+            netlists.append(QucsSettings.tempFilesDir.filePath("netlist.txt"));
         }
     }
 
@@ -848,26 +831,29 @@ void QucsApp::slotShowLastNetlist()
 // Is called to start the text editor.
 void QucsApp::slotCallEditor()
 {
-  editFile(QString(""));
+  editFile(QString());
 }
 
 // ------------------------------------------------------------------------
 // Is called to start the filter synthesis program.
 void QucsApp::slotCallFilter()
 {
-  launchTool(QUCS_NAME "filter", "filter synthesis");
+  auto currentStyle = QApplication::style()->objectName();
+  launchTool(QUCS_NAME "filter", "filter synthesis",(QStringList() << "-style" << currentStyle));
 }
 
 void QucsApp::slotCallActiveFilter()
 {
-  launchTool(QUCS_NAME "activefilter", "active filter synthesis");
+  auto currentStyle = QApplication::style()->objectName();
+  launchTool(QUCS_NAME "activefilter", "active filter synthesis",(QStringList() << "-style" << currentStyle));
 }
 
 // ------------------------------------------------------------------------
 // Is called to start the transmission line calculation program.
 void QucsApp::slotCallLine()
 {
-  launchTool(QUCS_NAME "trans", "line calculation",QStringList());
+  auto currentStyle = QApplication::style()->objectName();
+  launchTool(QUCS_NAME "trans", "line calculation",(QStringList() << "-style" << currentStyle));
 }
 
 // --------------------------------------------------------------
@@ -882,14 +868,23 @@ void QucsApp::slotCallMatch()
 // Is called to start the attenuator calculation program.
 void QucsApp::slotCallAtt()
 {
-  launchTool(QUCS_NAME "attenuator", "attenuator calculation",QStringList());
+  auto currentStyle = QApplication::style()->objectName();
+  launchTool(QUCS_NAME "attenuator", "attenuator calculation",(QStringList() << "-style" << currentStyle));
 }
 
 void QucsApp::slotCallPwrComb()
 {
-  launchTool(QUCS_NAME "powercombining", "power combining calculation",QStringList());
+  auto currentStyle = QApplication::style()->objectName();
+  launchTool(QUCS_NAME "powercombining", "power combining calculation",(QStringList() << "-style" << currentStyle));
 }
 
+void QucsApp::slotCallSPAR_Viewer()
+{
+  auto currentStyle = QApplication::style()->objectName();
+  launchTool(QUCS_NAME "spar-viewer", "s-parameter viewer",(QStringList() << "-style" << currentStyle));
+}
+
+
 /*!
  * \brief launch an external application passing arguments
  *
@@ -900,32 +895,40 @@ void QucsApp::slotCallPwrComb()
 void QucsApp::launchTool(const QString& prog, const QString& progDesc, const QStringList &args,
                          bool qucs_tool)
 {
-  QProcess *tool = new QProcess();
+    QString tooldir = qucs_tool ? QucsSettings.QucsatorDir : QucsSettings.BinDir;
 
-  QString tooldir;
-  if (qucs_tool) tooldir = QucsSettings.QucsatorDir;
-  else tooldir = QucsSettings.BinDir;
-#ifdef __MINGW32__
-  QString cmd = QDir::toNativeSeparators("\""+tooldir + prog + ".exe\"");
+    // Create command path based on the platform
+    QString cmd;
+#if defined(_WIN32) || defined(__MINGW32__)
+    cmd = QDir(tooldir).absoluteFilePath(prog + ".exe");
 #elif __APPLE__
-  QString cmd = QDir::toNativeSeparators(tooldir + prog + ".app/Contents/MacOS/" + prog);
+    cmd = QDir(tooldir).absoluteFilePath(prog + ".app/Contents/MacOS/" + prog);
 #else
-  QString cmd = QDir::toNativeSeparators(tooldir + prog);
+    cmd = QDir(tooldir).absoluteFilePath(prog);
 #endif
 
-  tool->setWorkingDirectory(tooldir);
-  qDebug() << "Command :" << cmd;
-  tool->start(cmd,args);
+    // Validate if the file exists before attempting to execute
+    if (!QFileInfo(cmd).exists()) {
+        QMessageBox::critical(this, tr("Error"),
+                            tr("Executable %1 not found! \n\n(%2)").arg(progDesc, cmd));
+        return;
+    }
 
-  if(!tool->waitForStarted(1000) ) {
-    QMessageBox::critical(this, tr("Error"),
-                          tr("Cannot start %1 program! \n\n(%2)").arg(progDesc, cmd));
-    delete tool;
-    return;
-  }
+    QProcess *tool = new QProcess();
 
-  // to kill the application first before qucs finishes exiting
-  connect(this, SIGNAL(signalKillEmAll()), tool, SLOT(kill()));
+    qDebug() << "Command :" << cmd;
+    tool->setWorkingDirectory(tooldir);
+    tool->start(cmd,args);
+
+    if(!tool->waitForStarted(1000) ) {
+        QMessageBox::critical(this, tr("Error"),
+                            tr("Cannot start %1 program! \n\n(%2)").arg(progDesc, cmd));
+        delete tool;
+        return;
+    }
+
+    // to kill the application first before qucs finishes exiting
+    connect(this, SIGNAL(signalKillEmAll()), tool, SLOT(kill()));
 }
 
 
@@ -939,8 +942,8 @@ void QucsApp::slotCallRFLayout()
                                   tr("Layouting of display pages is not supported!"));
             return;
         }
-        input_file = sch->DocName;
-        QFileInfo inf(sch->DocName);
+        input_file = sch->getDocName();
+        QFileInfo inf(sch->getDocName());
         odir = inf.absolutePath();
         netlist_file = inf.absolutePath() + QDir::separator()
                 + inf.baseName() + ".net";
@@ -953,7 +956,7 @@ void QucsApp::slotCallRFLayout()
         QStringList Collect;
         QPlainTextEdit *ErrText = new QPlainTextEdit();  //dummy
         int pNum = sch->prepareNetlist(stream, Collect, ErrText);
-        if (!sch->isAnalog) {
+        if (!sch->getIsAnalog()) {
             QMessageBox::critical(this, tr("Error"), tr("Digital schematic not supported!"));
             return;
         }
@@ -1043,7 +1046,7 @@ void QucsApp::slotAddToProject()
 
 
   QStringList List = QFileDialog::getOpenFileNames(this, tr("Select files to copy"),
-    lastDir.isEmpty() ? QString(".") : lastDir, QucsFileFilter);
+    lastDir.isEmpty() ? QStringLiteral(".") : lastDir, QucsFileFilter);
 
   if(List.isEmpty()) {
     statusBar()->showMessage(tr("No files copied."), 2000);
@@ -1124,9 +1127,19 @@ void QucsApp::slotCursorLeft(bool left)
   }
   if(!editText->isHidden()) return;  // for edit of component property ?
 
-  Q3PtrList<Element> movingElements;
+  QList<Element*> movingElements;
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
-  int markerCount = Doc->copySelectedElements(&movingElements);
+  int markerCount = 0;
+  {
+    // Q3PtrList is long time deprecated and has to be replaced with another
+    // container type, which is not always easy. Here it's simpler to use it
+    // once and go back to QList, because copySelectedElements() uses API
+    // unique to Q3PtrList and to refactor it is a piece of work
+    Q3PtrList<Element> temp_buffer;
+    temp_buffer.setAutoDelete(false);
+    markerCount = Doc->copySelectedElements(&temp_buffer);
+    for (auto* e : temp_buffer) { movingElements.append(e); }
+  }
 
   if((movingElements.count() - markerCount) < 1) {
     if(markerCount > 0) {  // only move marker if nothing else selected
@@ -1134,14 +1147,13 @@ void QucsApp::slotCursorLeft(bool left)
     } else if(left) {
       Doc->scrollLeft(Doc->horizontalScrollBar()->singleStep());
     }else{ // right
-      Doc->scrollRight(-Doc->horizontalScrollBar()->singleStep());
+      Doc->scrollRight(Doc->horizontalScrollBar()->singleStep());
     }
 
     Doc->viewport()->update();
-    view->drawn = false;
     return;
   } else { // random selection. move all of them
-    view->moveElements(&movingElements, sign*Doc->GridX, 0);
+    view->moveElements(&movingElements, sign*Doc->getGridX(), 0);
     view->MAx3 = 1;  // sign for moved elements
     view->endElementMoving(Doc, &movingElements);
   }
@@ -1191,9 +1203,19 @@ void QucsApp::slotCursorUp(bool up)
     return;
   }
 
-  Q3PtrList<Element> movingElements;
+  QList<Element*> movingElements;
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
-  int markerCount = Doc->copySelectedElements(&movingElements);
+  int markerCount = 0;
+  {
+    // Q3PtrList is long time deprecated and has to be replaced with another
+    // container type, which is not always easy. Here it's simpler to use it
+    // once and go back to QList, because copySelectedElements() uses API
+    // unique to Q3PtrList and to refactor it is a piece of work
+    Q3PtrList<Element> temp_buffer;
+    temp_buffer.setAutoDelete(false);
+    markerCount = Doc->copySelectedElements(&temp_buffer);
+    for (auto* e : temp_buffer) { movingElements.append(e); }
+  }
 
   if((movingElements.count() - markerCount) < 1) { // all selections are markers
     if(markerCount > 0) {  // only move marker if nothing else selected
@@ -1205,10 +1227,9 @@ void QucsApp::slotCursorUp(bool up)
     }
 
     Doc->viewport()->update();
-    view->drawn = false;
     return;
   }else{ // some random selection, put it back
-    view->moveElements(&movingElements, 0, ((up)?-1:1) * Doc->GridY);
+    view->moveElements(&movingElements, 0, ((up)?-1:1) * Doc->getGridY());
     view->MAx3 = 1;  // sign for moved elements
     view->endElementMoving(Doc, &movingElements);
   }
@@ -1220,110 +1241,94 @@ void QucsApp::slotCursorUp(bool up)
 // In "view->MAx3" is the number of the current property.
 void QucsApp::slotApplyCompText()
 {
-  QString s;
   QFont f = QucsSettings.font;
   Schematic *Doc = (Schematic*)DocumentTab->currentWidget();
-  f.setPointSizeF( Doc->Scale * float(f.pointSize()) );
+  f.setPointSizeF( Doc->getScale() * float(f.pointSize()) );
   editText->setFont(f);
 
-  Property  *pp = 0;
-  Component *pc = (Component*)view->focusElement;
-  if(!pc) return;  // should never happen
-  view->MAx1 = pc->cx + pc->tx;
-  view->MAy1 = pc->cy + pc->ty;
+  Component *const component = dynamic_cast<Component*>(view->focusElement);
+  if(!component) return;  // should never happen
+  view->MAx1 = component->cx + component->tx;
+  view->MAy1 = component->cy + component->ty;
 
-  int z, n=0;  // "n" is number of property on screen
-  pp = pc->Props.first();
-  for(z=view->MAx3; z>0; z--) {  // calculate "n"
-    if(!pp) {  // should never happen
-      slotHideEdit();
-      return;
-    }
-    if(pp->display) n++;   // is visible ?
-    pp = pc->Props.next();
-  }
+  // Here is a bit of *magic* and implicit coupling: the value of view->MAx3
+  // comes from Component::getTextSelected, and it's equal 0 when component
+  // name is clicked, N + 1 when Nth component property is clicked.
+  const int component_text_index = view->MAx3;
+  const bool is_name = component_text_index == 0;
 
-  pp = 0;
-  if(view->MAx3 > 0)  pp = pc->Props.at(view->MAx3-1); // current property
-  else s = pc->Name;
+  Property *const component_property = !is_name
+                                     ? component->Props.at(component_text_index - 1)
+                                     : nullptr;
 
-  if(!editText->isHidden()) {   // is called the first time ?
-    // no -> apply value to current property
-    if(view->MAx3 == 0) {   // component name ?
-      Component *pc2;
-      if(!editText->text().isEmpty())
-        if(pc->Name != editText->text()) {
-          for(pc2 = Doc->Components->first(); pc2!=0; pc2 = Doc->Components->next())
-            if(pc2->Name == editText->text())
-              break;  // found component with the same name ?
-          if(!pc2) {
-            pc->Name = editText->text();
-            Doc->setChanged(true, true);  // only one undo state
+  if (editText->isVisible()) {   // is called the first time ?
+    if (is_name) {
+      const auto new_name{editText->text()};
+
+      if (!new_name.isEmpty() && component->Name != new_name) {
+        // TODO: rewrite with std::none_of after replacing Q3PtrList
+        //       with modern container
+        bool is_unique = true;
+        for (auto* other : *Doc->a_Components) {
+          if (other->Name == new_name) {
+            is_unique = false;
+            break;
           }
         }
+
+        if (is_unique) {
+          component->Name = new_name;
+          Doc->setChanged(true, true);  // only one undo state
+        }
+      }
+
     }
-    else if(pp) {  // property was applied
-      if(pp->Value != editText->text()) {
-        pp->Value = editText->text();
-        Doc->recreateComponent(pc);  // because of "Num" and schematic symbol
+    else if (component_property) {  // property was applied
+      if (component_property->Value != editText->text()) {
+        component_property->Value = editText->text();
+        Doc->recreateComponent(component);  // because of "Num" and schematic symbol
         Doc->setChanged(true, true); // only one undo state
       }
     }
-
-    n++;     // next row on screen
-    (view->MAx3)++;  // next property
-    pp = pc->Props.at(view->MAx3-1);  // search for next property
-
-    Doc->viewport()->update();
-    view->drawn = false;
-
-    if(!pp) {     // was already last property ?
-      slotHideEdit();
-      return;
-    }
-
-
-    while(!pp->display) {  // search for next visible property
-      (view->MAx3)++;  // next property
-      pp = pc->Props.next();
-      if(!pp) {     // was already last property ?
-        slotHideEdit();
-        return;
-      }
-    }
   }
 
-  // avoid seeing the property text behind the line edit
-  if(pp)  // Is it first property or component name ?
-    s = pp->Value;
-  editText->setMinimumWidth(editText->fontMetrics().boundingRect(s).width()+4);
+  const QString s = is_name
+                ? component->Name
+                : component_property->Value;
 
-
-  Doc->contentsToViewport(int(Doc->Scale * float(view->MAx1 - Doc->ViewX1)),
-			 int(Doc->Scale * float(view->MAy1 - Doc->ViewY1)),
-			 view->MAx2, view->MAy2);
   editText->setReadOnly(false);
-  if(pp) {  // is it a property ?
-    s = pp->Value;
-    view->MAx2 += editText->fontMetrics().boundingRect(pp->Name+"=").width();
-    if(pp->Description.indexOf('[') >= 0)  // is selection list ?
+  QPoint editTextTopLeft;
+  if (component_property) {  // is it a property ?
+    editTextTopLeft = Doc->modelToViewport(QPoint{component->cx, component->cy} + component_property->boundingRect().topLeft());
+    editTextTopLeft.rx() += editText->fontMetrics().boundingRect(component_property->Name + "=" + '\u0020').width();
+
+    if(component_property->Description.indexOf('[') >= 0)  // is selection list ?
       editText->setReadOnly(true);
     Expr_CompProp.setPattern("[^\"]*");
-    if(!pc->showName) n--;
   }
-  else   // it is the component name
+  else { // it is the component name
     Expr_CompProp.setPattern("[\\w_]+");
+    editTextTopLeft = Doc->modelToViewport(QPoint{component->cx + component->tx, component->cy + component->ty});
+  }
+
+  {
+    auto size = editText->fontMetrics().boundingRect(s ).size();
+    size.rwidth() += editText->fontMetrics().averageCharWidth();
+    editText->setFixedSize(size);
+  }
+
+  view->MAx2 = editTextTopLeft.x();
+  view->MAy2 = editTextTopLeft.y();
+
   Val_CompProp.setRegularExpression(Expr_CompProp);
   editText->setValidator(&Val_CompProp);
 
-  z = editText->fontMetrics().lineSpacing();
-  view->MAy2 += n*z;
   editText->setText(s);
-  misc::setWidgetBackgroundColor(editText,QucsSettings.BGColor);
+  editText->setStyleSheet("color: black; background-color: " + QucsSettings.BGColor.name());
   editText->setFocus();
   editText->selectAll();
   editText->setParent(Doc->viewport());
-  editText->move(QPoint(view->MAx2, view->MAy2));
+  editText->move(view->MAx2, view->MAy2);
   editText->show();
   //editText->reparent(Doc->viewport(), 0, QPoint(view->MAx2, view->MAy2), true);
 }
@@ -1349,7 +1354,10 @@ void QucsApp::slotCreateLib()
 
   LibraryDialog *d = new LibraryDialog(this);
   d->fillSchematicList(Content->exportSchematic());
-  d->exec();
+  auto r = d->exec();
+  if (r != QDialog::Accepted) {
+    fillLibrariesTreeView();
+  }
 }
 
 // -----------------------------------------------------------
@@ -1364,10 +1372,10 @@ void QucsApp::slotImportData()
       QString dname;
       if (isTextDocument(DocumentTab->currentWidget())) {
           TextDoc *doc = (TextDoc *)DocumentTab->currentWidget();
-          dname = doc->DocName;
+          dname = doc->getDocName();
       } else {
           Schematic *doc = (Schematic *)DocumentTab->currentWidget();
-          dname = doc->DocName;
+          dname = doc->getDocName();
       }
       QFileInfo inf(dname);
       if (inf.exists()) {
@@ -1396,12 +1404,12 @@ void QucsApp::slotExportGraphAsCsv()
   }
 
   /*QString s = Q3FileDialog::getSaveFileName(
-     lastDir.isEmpty() ? QString(".") : lastDir,
+     lastDir.isEmpty() ? QStringLiteral(".") : lastDir,
      tr("CSV file")+" (*.csv);;" + tr("Any File")+" (*)",
      this, 0, tr("Enter an Output File Name"));
      */
   QString s = QFileDialog::getSaveFileName(this, tr("Enter an Output File Name"),
-    lastDir.isEmpty() ? QString(".") : lastDir, tr("CSV file")+" (*.csv);;" + tr("Any File")+" (*)");
+    lastDir.isEmpty() ? QStringLiteral(".") : lastDir, tr("CSV file")+" (*.csv);;" + tr("Any File")+" (*)");
 
   if(s.isEmpty())
     return;
@@ -1440,7 +1448,7 @@ void QucsApp::slotExportGraphAsCsv()
   int Count = g->countY * g->axis(0)->count;
   for(n = 0; n < Count; n++) {
     m = n;
-	 for(unsigned ii=0; (pD=g->axis(ii)); ++ii) {
+    for(unsigned ii=0; (pD=g->axis(ii)); ++ii) {
       Stream << *(pD->Points + m%pD->count) << ';';
       m /= pD->count;
     }
@@ -1452,24 +1460,6 @@ void QucsApp::slotExportGraphAsCsv()
   File.close();
 }
 
-// ----------------------------------------------------------
-void QucsApp::slotCreatePackage()
-{
-  slotHideEdit(); // disable text edit of component property
-
-  PackageDialog *d = new PackageDialog(this, true);
-  d->exec();
-}
-
-// ----------------------------------------------------------
-void QucsApp::slotExtractPackage()
-{
-  slotHideEdit(); // disable text edit of component property
-  PackageDialog *d = new PackageDialog(this, false);
-  d->show();
-  d->extractPackage();
-  readProjects();
-}
 
 void QucsApp::slotOpenRecent()
 {
@@ -1630,7 +1620,7 @@ void QucsApp::slotBuildModule()
 
     QString make;
 
-#ifdef __MINGW32__
+#if defined(_WIN32) || defined(__MINGW32__)
     make = "mingw32-make.exe";    // must be on the path!
 #else
     make = "make";                // must be on the path!
@@ -1652,11 +1642,11 @@ void QucsApp::slotBuildModule()
     builder->setProcessChannelMode(QProcess::MergedChannels);
     // get current va document
     QucsDoc *Doc = getDoc();
-    QString vaModule = Doc->fileBase(Doc->DocName);
+    QString vaModule = Doc->fileBase(Doc->getDocName());
 
     QString admsXml = QucsSettings.AdmsXmlBinDir.canonicalPath();
 
-#ifdef __MINGW32__
+#if defined(_WIN32) || defined(__MINGW32__)
     admsXml = QDir::toNativeSeparators(admsXml+"/"+"admsXml.exe");
 #else
     admsXml = QDir::toNativeSeparators(admsXml+"/"+"admsXml");
@@ -1665,16 +1655,16 @@ void QucsApp::slotBuildModule()
     // admsXml emits C++
     QStringList Arguments;
     Arguments << "-f" <<  QDir::toNativeSeparators(include.absoluteFilePath("va2cpp.makefile"))
-              << QString("ADMSXML=%1").arg(admsXml)
-              << QString("PREFIX=%1").arg(QDir::toNativeSeparators(prefix.absolutePath()))
-              << QString("MODEL=%1").arg(vaModule);
+              << QStringLiteral("ADMSXML=%1").arg(admsXml)
+              << QStringLiteral("PREFIX=%1").arg(QDir::toNativeSeparators(prefix.absolutePath()))
+              << QStringLiteral("MODEL=%1").arg(vaModule);
 
     QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
     env.insert("PATH", env.value("PATH") );
     builder->setProcessEnvironment(env);
 
     // prepend command to log
-    QString cmdString = QString("%1 %2\n").arg(make, Arguments.join(" "));
+    QString cmdString = QStringLiteral("%1 %2\n").arg(make, Arguments.join(" "));
     messageDock->admsOutput->appendPlainText(cmdString);
 
     qDebug() << "Command :" << make << Arguments.join(" ");
@@ -1697,12 +1687,12 @@ void QucsApp::slotBuildModule()
     Arguments.clear();
 
     Arguments << "-f" <<  QDir::toNativeSeparators(include.absoluteFilePath("cpp2lib.makefile"))
-              << QString("PREFIX=\"%1\"").arg(QDir::toNativeSeparators(prefix.absolutePath()))
-              << QString("PROJDIR=\"%1\"").arg(QDir::toNativeSeparators(workDir))
-              << QString("MODEL=%1").arg(vaModule);
+              << QStringLiteral("PREFIX=\"%1\"").arg(QDir::toNativeSeparators(prefix.absolutePath()))
+              << QStringLiteral("PROJDIR=\"%1\"").arg(QDir::toNativeSeparators(workDir))
+              << QStringLiteral("MODEL=%1").arg(vaModule);
 
     // prepend command to log
-    cmdString = QString("%1 %2\n").arg(make, Arguments.join(" "));
+    cmdString = QStringLiteral("%1 %2\n").arg(make, Arguments.join(" "));
     messageDock->cppOutput->appendPlainText(cmdString);
 
     builder->start(make, Arguments);
@@ -1743,7 +1733,7 @@ void QucsApp::buildWithOpenVAF()
     builder->setProcessChannelMode(QProcess::MergedChannels);
     // get current va document
     QucsDoc *Doc = getDoc();
-    QString vaModule = Doc->DocName;
+    QString vaModule = Doc->getDocName();
 
     QString openVAF = QucsSettings.OpenVAFExecutable;
 
@@ -1756,7 +1746,7 @@ void QucsApp::buildWithOpenVAF()
     builder->setProcessEnvironment(env);
 
     // prepend command to log
-    QString cmdString = QString("%1 %2\n").arg(openVAF, Arguments.join(" "));
+    QString cmdString = QStringLiteral("%1 %2\n").arg(openVAF, Arguments.join(" "));
     messageDock->admsOutput->appendPlainText(cmdString);
 
     qDebug() << "Command :" << openVAF << Arguments.join(" ");
