@@ -22,6 +22,7 @@
 #include "main.h"
 #include "misc.h"
 #include "qucs.h"
+#include "extsimkernels/spicecompat.h"
 #include "octave_window.h"
 
 #include <QAction>
@@ -58,6 +59,12 @@ void QucsApp::initActions()
   textNew->setWhatsThis(tr("New Text\n\nCreates a new text document"));
   connect(textNew, SIGNAL(triggered()), SLOT(slotTextNew()));
 
+  symNew = new QAction(QIcon((":/bitmaps/svg/symnew.svg")), tr("New symbol"), this);
+  symNew->setStatusTip(tr("Creates a new symbol"));
+  symNew->setWhatsThis(tr("New\n\nCreates a new schematic symbol document"));
+  connect(symNew, SIGNAL(triggered()), SLOT(slotSymbolNew()));
+
+
   fileOpen = new QAction(QIcon((":/bitmaps/fileopen.png")),	tr("&Open..."), this);
   fileOpen->setShortcut(QKeySequence::Open);
   fileOpen->setStatusTip(tr("Opens an existing document"));
@@ -83,7 +90,7 @@ void QucsApp::initActions()
   connect(fileSaveAll, SIGNAL(triggered()), SLOT(slotFileSaveAll()));
 
   fileClose = new QAction(QIcon((":/bitmaps/svg/fileclose.svg")), tr("&Close"), this);
-  fileClose->setShortcut(QKeySequence::Close);
+  fileClose->setShortcut(Qt::CTRL|Qt::Key_E);
   fileClose->setStatusTip(tr("Closes the current document"));
   fileClose->setWhatsThis(tr("Close File\n\nCloses the current document"));
   connect(fileClose, SIGNAL(triggered()), SLOT(slotFileClose()));
@@ -98,9 +105,9 @@ void QucsApp::initActions()
   connect(fileClearRecent, SIGNAL(triggered()), SLOT(slotClearRecentFiles()));
 
   fileExamples = new QAction(tr("&Examples"), this);
-  fileExamples->setStatusTip(tr("Opens a file explorer with example documents"));
+  fileExamples->setStatusTip(tr("Starts file chooser dialog to open one of example schematics"));
   fileExamples->setWhatsThis(
-	        tr("Examples\n\nOpens a file explorer with example documents"));
+	        tr("Examples\n\nStart file chooser dialog and open one of example schematics"));
   connect(fileExamples, SIGNAL(triggered()), SLOT(slotFileExamples()));
 
 
@@ -240,7 +247,11 @@ void QucsApp::initActions()
   connect(editPaste, SIGNAL(toggled(bool)), SLOT(slotEditPaste(bool)));
 
   editDelete = new QAction(QIcon((":/bitmaps/svg/editdelete.svg")), tr("&Delete"), this);
+#ifdef __APPLE__
+  editDelete->setShortcuts({QKeySequence(Qt::Key_Backspace), QKeySequence(Qt::Key_Delete)});
+#else
   editDelete->setShortcut(QKeySequence::Delete);
+#endif  
   editDelete->setStatusTip(tr("Deletes the selected components"));
   editDelete->setWhatsThis(tr("Delete\n\nDeletes the selected components"));
   editDelete->setCheckable(true);
@@ -324,18 +335,6 @@ void QucsApp::initActions()
 	tr("Create Library\n\nCreate Library from Subcircuits"));
   connect(createLib, SIGNAL(triggered()), SLOT(slotCreateLib()));
 
-  createPkg = new QAction(tr("Create &Package..."), this);
-  createPkg->setShortcut(tr("Ctrl+Shift+Y"));
-  createPkg->setStatusTip(tr("Create compressed Package from Projects"));
-  createPkg->setWhatsThis(tr("Create Package\n\nCreate compressed Package from complete Projects"));
-  connect(createPkg, SIGNAL(triggered()), SLOT(slotCreatePackage()));
-
-  extractPkg = new QAction(tr("E&xtract Package..."),  this);
-  extractPkg->setShortcut(tr("Ctrl+Shift+X"));
-  extractPkg->setStatusTip(tr("Install Content of a Package"));
-  extractPkg->setWhatsThis(tr("Extract Package\n\nInstall Content of a Package"));
-  connect(extractPkg, SIGNAL(triggered()), SLOT(slotExtractPackage()));
-
   graph2csv = new QAction(tr("Export to &CSV..."), this);
   graph2csv->setShortcut(tr("Ctrl+Shift+C"));
   graph2csv->setStatusTip(tr("Convert graph data to CSV file"));
@@ -371,14 +370,14 @@ void QucsApp::initActions()
   connect(magOne, SIGNAL(triggered()), SLOT(slotShowOne()));
 
   magPlus = new QAction(QIcon((":/bitmaps/svg/viewmag+.svg")),	tr("Zoom in"), this);
-  magPlus->setShortcut(Qt::Key_Plus);
+  magPlus->setShortcut(QKeySequence::ZoomIn);
   magPlus->setStatusTip(tr("Zooms into the current view"));
   magPlus->setWhatsThis(tr("Zoom in\n\nZooms the current view"));
   magPlus->setCheckable(true);
   connect(magPlus, SIGNAL(toggled(bool)), SLOT(slotZoomIn(bool)));
 
   magMinus = new QAction(QIcon((":/bitmaps/svg/viewmag-.svg")), tr("Zoom out"), this);
-  magMinus->setShortcut(Qt::Key_Minus);
+  magMinus->setShortcut(QKeySequence::ZoomOut);
   magMinus->setStatusTip(tr("Zooms out the current view"));
   magMinus->setWhatsThis(tr("Zoom out\n\nZooms out the current view"));
   connect(magMinus, SIGNAL(triggered()), SLOT(slotZoomOut()));
@@ -531,6 +530,11 @@ void QucsApp::initActions()
   callPwrComb->setWhatsThis(tr("Power combining\n\nStarts power combining calculation program"));
   connect(callPwrComb, SIGNAL(triggered()), SLOT(slotCallPwrComb()));
 
+  callSPAR_Viewer = new QAction(tr("S-parameter Viewer"), this);
+  callSPAR_Viewer->setStatusTip(tr("Starts S-parameter viewer"));
+  callSPAR_Viewer->setWhatsThis(tr("S-parameter Viewer\n\nStarts S-parameter viewer"));
+  connect(callSPAR_Viewer, SIGNAL(triggered()), SLOT(slotCallSPAR_Viewer()));
+
   callConverter = new QAction(tr("Data files converter"), this);
   callConverter->setShortcut(tr("Ctrl+8"));
   callConverter->setStatusTip(tr("Convert data file"));
@@ -549,10 +553,10 @@ void QucsApp::initActions()
   simulate->setWhatsThis(tr("Simulate\n\nSimulates the current schematic"));
   connect(simulate, SIGNAL(triggered()), SLOT(slotSimulate()));
 
-  tune = new QAction(QIcon((":/bitmaps/svg/tune.svg")),"Tune", this);
+  tune = new QAction(QIcon((":/bitmaps/svg/tune.svg")),tr("Tune"), this);
   tune->setShortcut(Qt::Key_F3);
-  tune->setStatusTip("Tuner");
-  tune->setWhatsThis("Allows to live tune variables and show the result in the dataview");
+  tune->setStatusTip(tr("Tuner"));
+  tune->setWhatsThis(tr("Allows to live tune variables and show the result in the dataview"));
   tune->setCheckable(true);
   connect(tune, SIGNAL(toggled(bool)), SLOT(slotTune(bool)));
 
@@ -570,9 +574,14 @@ void QucsApp::initActions()
 
   save_netlist = new QAction(tr("Save netlist"), this);
   save_netlist->setStatusTip(tr("Save netlist"));
-  save_netlist->setWhatsThis(tr("Save netlist to file"));
+  save_netlist->setWhatsThis(tr(QString::fromUtf8("Save netlist to %1").arg(a_netlist2Console ? "console" : "file").toLatin1().constData()));
   connect(save_netlist, SIGNAL(triggered()), SLOT(slotSaveNetlist()));
 
+  saveCdlNetlist = new QAction(tr("Save CDL netlist"), this);
+  saveCdlNetlist->setStatusTip(tr("Save CDL netlist"));
+  saveCdlNetlist->setWhatsThis(tr(QString::fromUtf8("Save CDL netlist to %1").arg(a_netlist2Console ? "console" : "file").toLatin1().constData()));
+  connect(saveCdlNetlist, SIGNAL(triggered()), SLOT(slotSaveCdlNetlist()));
+
   setMarker = new QAction(QIcon((":/bitmaps/svg/marker.svg")),	tr("Set Marker on Graph"), this);
   setMarker->setShortcut(Qt::CTRL|Qt::Key_B);
   setMarker->setStatusTip(tr("Sets a marker on a diagram's graph"));
@@ -581,7 +590,7 @@ void QucsApp::initActions()
   connect(setMarker, SIGNAL(toggled(bool)), SLOT(slotSetMarker(bool)));
 
   setDiagramLimits = new QAction(QIcon((":/bitmaps/svg/viewwave.svg")),	tr("Set Diagram Limits"), this);
-  setDiagramLimits->setShortcut(tr("Ctrl+E"));
+  //setDiagramLimits->setShortcut(tr("Ctrl+E"));
   setDiagramLimits->setStatusTip(tr("Pick the diagram limits using the mouse. Right click for default."));
   setDiagramLimits->setWhatsThis(tr("Set Diagram Limits\n\nPick the diagram limits using the mouse. Right click for default."));
   setDiagramLimits->setCheckable(true);
@@ -593,6 +602,13 @@ void QucsApp::initActions()
   resetDiagramLimits->setWhatsThis(tr("Reset Diagram Limits\n\nResets the limits for all axis to auto."));
   connect(resetDiagramLimits, SIGNAL(triggered()), SLOT(slotResetDiagramLimits()));
 
+  showGrid = new QAction(tr("Show Grid (current document)"), this);
+  showGrid->setCheckable(true);
+  showGrid->setShortcut(tr("Alt+G"));
+  showGrid->setStatusTip(tr("Show or hide the grid for the current document."));
+  showGrid->setWhatsThis(tr("Show / Hide Grid\n\nShow or hide the grid for the current document."));
+  connect(showGrid, SIGNAL(triggered()), SLOT(slotShowGrid()));
+
   showMsg = new QAction(tr("Show Last Messages"), this);
   showMsg->setShortcut(Qt::Key_F5);
   showMsg->setStatusTip(tr("Shows last simulation messages"));
@@ -612,19 +628,6 @@ void QucsApp::initActions()
   //buildIFS = new QAction(tr("Build XSPICE IFS file from subcircuit"),this);
   //connect(buildIFS,SIGNAL(triggered()),SLOT(slotBuildXSPICEIfs()));
 
-
-  viewToolBar = new QAction(tr("Tool&bar"), this);
-  viewToolBar->setCheckable(true);
-  viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
-  viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
-  connect(viewToolBar, SIGNAL(toggled(bool)), SLOT(slotViewToolBar(bool)));
-
-  viewStatusBar = new QAction(tr("&Statusbar"), this);
-  viewStatusBar->setCheckable(true);
-  viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
-  viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
-  connect(viewStatusBar, SIGNAL(toggled(bool)), SLOT(slotViewStatusBar(bool)));
-
   viewBrowseDock = new QAction(tr("&Dock Window"), this);
   viewBrowseDock->setCheckable(true);
   viewBrowseDock->setStatusTip(tr("Enables/disables the browse dock window"));
@@ -653,12 +656,12 @@ void QucsApp::initActions()
   helpGetStart->setWhatsThis(tr("Getting Started\n\nShort introduction into Qucs"));
   connect(helpGetStart, SIGNAL(triggered()), SLOT(slotGettingStarted()));
 
-  helpAboutApp = new QAction(tr("&About Qucs-S..."), this);
+  helpAboutApp = new QAction(tr("&About Qucs-S"), this);
   helpAboutApp->setStatusTip(tr("About the application"));
   helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
-  connect(helpAboutApp, SIGNAL(triggered()), SLOT(slotHelpAbout()));
+  connect(helpAboutApp, SIGNAL(triggered()),this, SLOT(slotHelpAbout()));
 
-  helpAboutQt = new QAction(tr("About Qt..."), this);
+  helpAboutQt = new QAction(tr("&About Qt"), this);
   helpAboutQt->setStatusTip(tr("About Qt"));
   helpAboutQt->setWhatsThis(tr("About Qt\n\nAbout Qt by Trolltech"));
   connect(helpAboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
@@ -670,6 +673,7 @@ void QucsApp::initMenuBar()
   fileMenu = new QMenu(tr("&File"));  // menuBar entry fileMenu
   fileMenu->addAction(fileNew);
   fileMenu->addAction(textNew);
+  fileMenu->addAction(symNew);
   fileMenu->addAction(fileOpen);
   fileMenu->addAction(fileClose);
 
@@ -758,8 +762,6 @@ void QucsApp::initMenuBar()
   projMenu->addAction(projDel);
   projMenu->addSeparator();
   projMenu->addAction(createLib);
-  projMenu->addAction(createPkg);
-  projMenu->addAction(extractPkg);
   projMenu->addSeparator();
   projMenu->addAction(graph2csv);
   // TODO only enable if document is VA file
@@ -781,6 +783,7 @@ void QucsApp::initMenuBar()
   toolMenu->addAction(callPwrComb);
   toolMenu->addAction(callConverter);
   toolMenu->addAction(callRFLayout);
+  toolMenu->addAction(callSPAR_Viewer);
   toolMenu->addSeparator();
 
   cmMenu = new QMenu(tr("Compact modelling"));
@@ -798,6 +801,7 @@ void QucsApp::initMenuBar()
   simMenu->addAction(showMsg);
   simMenu->addAction(showNet);
   simMenu->addAction(save_netlist);
+  simMenu->addAction(saveCdlNetlist);
   simMenu->addAction(simSettings);
 
 
@@ -809,9 +813,9 @@ void QucsApp::initMenuBar()
   viewMenu->addAction(magMinus);
   viewMenu->addAction(setDiagramLimits);
   viewMenu->addSeparator();
-  //viewMenu->setCheckable(true);
-  viewMenu->addAction(viewToolBar);
-  viewMenu->addAction(viewStatusBar);
+  viewMenu->addAction(showGrid);
+  viewMenu->addSeparator();
+    //viewMenu->setCheckable(true);
   viewMenu->addAction(viewBrowseDock);
   viewMenu->addAction(viewOctaveDock);
 
@@ -909,6 +913,7 @@ void QucsApp::initToolBar()
   this->addToolBar(fileToolbar);
   fileToolbar->addAction(fileNew);
   fileToolbar->addAction(textNew);
+  fileToolbar->addAction(symNew);
   fileToolbar->addAction(fileOpen);
   fileToolbar->addAction(fileSave);
   fileToolbar->addAction(fileSaveAll);
@@ -1042,24 +1047,6 @@ void QucsApp::slotUpdateRedo(bool isEnabled)
   redo->setEnabled(isEnabled);
 }
 
-// ----------------------------------------------------------
-// turn Toolbar on or off
-void QucsApp::slotViewToolBar(bool toggle)
-{
-  fileToolbar->setVisible(toggle);
-  editToolbar->setVisible(toggle);
-  viewToolbar->setVisible(toggle);
-  workToolbar->setVisible(toggle);
-  simulateToolbar->setVisible(toggle);
-}
-
-// ----------------------------------------------------------
-// turn Statusbar on or off
-void QucsApp::slotViewStatusBar(bool toggle)
-{
-  statusBar()->setVisible(toggle);
-}
-
 // ----------------------------------------------------------
 // turn Browse Dock Window on or off
 void QucsApp::slotViewBrowseDock(bool toggle)
