@@ -18,10 +18,10 @@
 #ifndef SP_XYCESCRIPT_H
 #define SP_XYCESCRIPT_H
 
-#include "components/component.h"
+#include "components/simulation.h"
 
 
-class XyceScript : public Component  {
+class XyceScript : public qucs::component::SimulationComponent  {
 public:
   XyceScript();
   ~XyceScript();
@@ -29,7 +29,8 @@ public:
   static Element* info(QString&, char* &, bool getNewOne=false);
 
 protected:
-  QString spice_netlist(bool isXyce = false);
+  QString spice_netlist(spicecompat::SpiceDialect dialect = spicecompat::SPICEDefault);
+  Qt::GlobalColor color() const override { return Qt::darkGreen; }
 };
 
 #endif
